// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.WindowsAzure.Management.Compute.Models;

namespace Microsoft.WindowsAzure.Management.Compute.Models
{
    /// <summary>
    /// The set of parameters required to update a load balanced endpoint set.
    /// </summary>
    public partial class VirtualMachineUpdateLoadBalancedSetParameters
    {
        private IList<VirtualMachineUpdateLoadBalancedSetParameters.InputEndpoint> _loadBalancedEndpoints;
        
        /// <summary>
        /// Optional. A list of load balanced InputEndpoints to update.
        /// </summary>
        public IList<VirtualMachineUpdateLoadBalancedSetParameters.InputEndpoint> LoadBalancedEndpoints
        {
            get { return this._loadBalancedEndpoints; }
            set { this._loadBalancedEndpoints = value; }
        }
        
        /// <summary>
        /// Initializes a new instance of the
        /// VirtualMachineUpdateLoadBalancedSetParameters class.
        /// </summary>
        public VirtualMachineUpdateLoadBalancedSetParameters()
        {
            this._loadBalancedEndpoints = new List<VirtualMachineUpdateLoadBalancedSetParameters.InputEndpoint>();
        }
        
        /// <summary>
        /// The modeled external endpoint for a persistent VM role.
        /// </summary>
        public partial class InputEndpoint
        {
            private bool? _enableDirectServerReturn;
            
            /// <summary>
            /// Optional. A Boolean specifying whether this endpoint uses
            /// Direct Server Return
            /// </summary>
            public bool? EnableDirectServerReturn
            {
                get { return this._enableDirectServerReturn; }
                set { this._enableDirectServerReturn = value; }
            }
            
            private string _loadBalancedEndpointSetName;
            
            /// <summary>
            /// Required. Specifies whether this endpoint is part of shared
            /// LoadBalanced endpoint and served by multiple role instances.
            /// If not specified a BadRequest error will be returned. It must
            /// also be in use by the deployment (at least one role in the
            /// deployment must have an endpoint whose
            /// LoadBalancedEndpointSetName   matches this) otherwise a
            /// BadRequest error will be returned.
            /// </summary>
            public string LoadBalancedEndpointSetName
            {
                get { return this._loadBalancedEndpointSetName; }
                set { this._loadBalancedEndpointSetName = value; }
            }
            
            private LoadBalancerProbe _loadBalancerProbe;
            
            /// <summary>
            /// Optional. This represents an endpoint setting which platform
            /// load balancer must monitor to detect the availability of this
            /// role before forwarding traffic to this endpoint. If not
            /// specified the probe settings (if any) from the existing load
            /// balanced endpoint definition  will be retained.
            /// </summary>
            public LoadBalancerProbe LoadBalancerProbe
            {
                get { return this._loadBalancerProbe; }
                set { this._loadBalancerProbe = value; }
            }
            
            private int? _localPort;
            
            /// <summary>
            /// Optional. Specifies the internal port on which a service
            /// running inside the VM is listening to serve this endpoint.
            /// WARNING: If specified then ALL the endpoints of this LB set on
            /// all the roles will be updated to have THIS local port. To keep
            /// unique local ports on each role for a load balanced endpoint
            /// specify this as 0 (zero) and if you need to change those use
            /// UpdateRole. In case of port conflict with a local port (or
            /// probe port) on a role a BadRequestwill be returned.
            /// </summary>
            public int? LocalPort
            {
                get { return this._localPort; }
                set { this._localPort = value; }
            }
            
            private string _name;
            
            /// <summary>
            /// Optional.  The name of the InputEndpoint.  The name is ignored
            /// if specified
            /// </summary>
            public string Name
            {
                get { return this._name; }
                set { this._name = value; }
            }
            
            private int? _port;
            
            /// <summary>
            /// Optional. An integer specifying the public port for this
            /// endpoint. Allowed values are between 1 and 65535 inclusive. A
            /// unqiue Port and Protocol combination must be specified for
            /// each InputEndpoint in the list.
            /// </summary>
            public int? Port
            {
                get { return this._port; }
                set { this._port = value; }
            }
            
            private string _protocol;
            
            /// <summary>
            /// Optional. Specifies the transport protocol for the endpoint.
            /// </summary>
            public string Protocol
            {
                get { return this._protocol; }
                set { this._protocol = value; }
            }
            
            private IList<AccessControlListRule> _rules;
            
            /// <summary>
            /// Optional. A collection of access control rules which control
            /// the external network traffic reaching to this endpoint. NOTES:
            /// (1) To remove the ACLs from a load-balanced endpoint just omit
            /// this element. (2) ACLs are set as specified. There is no merge
            /// done with existing ACLs.
            /// </summary>
            public IList<AccessControlListRule> Rules
            {
                get { return this._rules; }
                set { this._rules = value; }
            }
            
            private string _virtualIPAddress;
            
            /// <summary>
            /// Optional. The virtual IP address of the endpoint.
            /// </summary>
            public string VirtualIPAddress
            {
                get { return this._virtualIPAddress; }
                set { this._virtualIPAddress = value; }
            }
            
            /// <summary>
            /// Initializes a new instance of the InputEndpoint class.
            /// </summary>
            public InputEndpoint()
            {
                this._rules = new List<AccessControlListRule>();
            }
        }
    }
}
