// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.Management.Sql;
using Microsoft.WindowsAzure.Management.Sql.Models;

namespace Microsoft.WindowsAzure
{
    /// <summary>
    /// The SQL Database Management API is a REST API for managing SQL Database
    /// servers and the firewall rules associated with SQL Database servers.
    /// (see
    /// http://msdn.microsoft.com/en-us/library/windowsazure/gg715283.aspx for
    /// more information)
    /// </summary>
    public static partial class FirewallRuleOperationsExtensions
    {
        /// <summary>
        /// Adds a new server-level firewall rule for a SQL Database server
        /// that belongs to a subscription.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/gg715280.aspx
        /// for more information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Sql.IFirewallRuleOperations.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the SQL database server to which this rule
        /// will be applied.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters for the Create Firewall Rule operation.
        /// </param>
        /// <returns>
        /// Response containing the firewall rule create response.
        /// </returns>
        public static FirewallRuleCreateResponse Create(this IFirewallRuleOperations operations, string serverName, FirewallRuleCreateParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IFirewallRuleOperations)s).CreateAsync(serverName, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Adds a new server-level firewall rule for a SQL Database server
        /// that belongs to a subscription.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/gg715280.aspx
        /// for more information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Sql.IFirewallRuleOperations.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the SQL database server to which this rule
        /// will be applied.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters for the Create Firewall Rule operation.
        /// </param>
        /// <returns>
        /// Response containing the firewall rule create response.
        /// </returns>
        public static Task<FirewallRuleCreateResponse> CreateAsync(this IFirewallRuleOperations operations, string serverName, FirewallRuleCreateParameters parameters)
        {
            return operations.CreateAsync(serverName, parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// Deletes a server-level firewall rule from a SQL Database server
        /// that belongs to a subscription.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/gg715277.aspx
        /// for more information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Sql.IFirewallRuleOperations.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the server that will be have new firewall
        /// rule applied to it.
        /// </param>
        /// <param name='ruleName'>
        /// Required. The name of the new firewall rule.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static OperationResponse Delete(this IFirewallRuleOperations operations, string serverName, string ruleName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IFirewallRuleOperations)s).DeleteAsync(serverName, ruleName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Deletes a server-level firewall rule from a SQL Database server
        /// that belongs to a subscription.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/gg715277.aspx
        /// for more information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Sql.IFirewallRuleOperations.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the server that will be have new firewall
        /// rule applied to it.
        /// </param>
        /// <param name='ruleName'>
        /// Required. The name of the new firewall rule.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<OperationResponse> DeleteAsync(this IFirewallRuleOperations operations, string serverName, string ruleName)
        {
            return operations.DeleteAsync(serverName, ruleName, CancellationToken.None);
        }
        
        /// <summary>
        /// Returns a list of all the server-level firewall rules for a SQL
        /// Database server that belongs to a subscription.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/gg715278.aspx
        /// for more information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Sql.IFirewallRuleOperations.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the server for which the call is being made.
        /// </param>
        /// <param name='ruleName'>
        /// Required. The name of the rule for which the call is being made.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static FirewallRuleGetResponse Get(this IFirewallRuleOperations operations, string serverName, string ruleName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IFirewallRuleOperations)s).GetAsync(serverName, ruleName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Returns a list of all the server-level firewall rules for a SQL
        /// Database server that belongs to a subscription.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/gg715278.aspx
        /// for more information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Sql.IFirewallRuleOperations.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the server for which the call is being made.
        /// </param>
        /// <param name='ruleName'>
        /// Required. The name of the rule for which the call is being made.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<FirewallRuleGetResponse> GetAsync(this IFirewallRuleOperations operations, string serverName, string ruleName)
        {
            return operations.GetAsync(serverName, ruleName, CancellationToken.None);
        }
        
        /// <summary>
        /// Returns a list of all the server-level firewall rules for a SQL
        /// Database server that belongs to a subscription.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/gg715278.aspx
        /// for more information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Sql.IFirewallRuleOperations.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the server for which the call is being made.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static FirewallRuleListResponse List(this IFirewallRuleOperations operations, string serverName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IFirewallRuleOperations)s).ListAsync(serverName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Returns a list of all the server-level firewall rules for a SQL
        /// Database server that belongs to a subscription.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/gg715278.aspx
        /// for more information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Sql.IFirewallRuleOperations.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the server for which the call is being made.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<FirewallRuleListResponse> ListAsync(this IFirewallRuleOperations operations, string serverName)
        {
            return operations.ListAsync(serverName, CancellationToken.None);
        }
        
        /// <summary>
        /// Updates an existing server-level firewall rule for a SQL Database
        /// server that belongs to a subscription.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/gg715280.aspx
        /// for more information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Sql.IFirewallRuleOperations.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the SQL database server to which this rule
        /// will be applied.
        /// </param>
        /// <param name='ruleName'>
        /// Required. The name of the firewall rule to be updated.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters for the Update Firewall Rule operation.
        /// </param>
        /// <returns>
        /// Response containing the firewall rule update response.
        /// </returns>
        public static FirewallRuleUpdateResponse Update(this IFirewallRuleOperations operations, string serverName, string ruleName, FirewallRuleUpdateParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IFirewallRuleOperations)s).UpdateAsync(serverName, ruleName, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Updates an existing server-level firewall rule for a SQL Database
        /// server that belongs to a subscription.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/gg715280.aspx
        /// for more information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Sql.IFirewallRuleOperations.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the SQL database server to which this rule
        /// will be applied.
        /// </param>
        /// <param name='ruleName'>
        /// Required. The name of the firewall rule to be updated.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters for the Update Firewall Rule operation.
        /// </param>
        /// <returns>
        /// Response containing the firewall rule update response.
        /// </returns>
        public static Task<FirewallRuleUpdateResponse> UpdateAsync(this IFirewallRuleOperations operations, string serverName, string ruleName, FirewallRuleUpdateParameters parameters)
        {
            return operations.UpdateAsync(serverName, ruleName, parameters, CancellationToken.None);
        }
    }
}
