// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Collections.Generic;
using System.Linq;

namespace Microsoft.WindowsAzure.Management.Storage.Models
{
    /// <summary>
    /// Parameters supplied to the Create Storage Account operation.
    /// </summary>
    public partial class StorageAccountCreateParameters
    {
        private string _affinityGroup;
        
        /// <summary>
        /// Optional. The name of an existing affinity group in the specified
        /// subscription. Required if Location is not specified. You can
        /// include either a Location or AffinityGroup element in the request
        /// body, but not both. To list available affinity groups, use the
        /// List Affinity Groups operation.
        /// </summary>
        public string AffinityGroup
        {
            get { return this._affinityGroup; }
            set { this._affinityGroup = value; }
        }
        
        private string _description;
        
        /// <summary>
        /// Optional. A description for the storage account. The description
        /// may be up to 1024 characters in length.
        /// </summary>
        public string Description
        {
            get { return this._description; }
            set { this._description = value; }
        }
        
        private IDictionary<string, string> _extendedProperties;
        
        /// <summary>
        /// Optional. Represents the name of an extended storage account
        /// property. Each extended property must have a defined name and a
        /// value. You can have a maximum of 50 extended property name/value
        /// pairs. The maximum length of the Name element is 64 characters,
        /// only alphanumeric characters and underscores are valid in the
        /// Name, and the name must start with a letter. Attempting to use
        /// other characters, starting the Name with a non-letter character,
        /// or entering a name that is identical to that of another extended
        /// property owned by the same storage account will result in a status
        /// code 400 (Bad Request) error. Each extended property value has a
        /// maximum length of 255 characters.
        /// </summary>
        public IDictionary<string, string> ExtendedProperties
        {
            get { return this._extendedProperties; }
            set { this._extendedProperties = value; }
        }
        
        private bool _geoReplicationEnabled;
        
        /// <summary>
        /// Optional. Specifies whether the storage account is created with
        /// geo-replication enabled. If the element is not included in the
        /// request body, the default value is true. If set to true, the data
        /// in the storage account is replicated across more than one
        /// geographic location to enable resilience in the face of
        /// catastrophic service loss.
        /// </summary>
        public bool GeoReplicationEnabled
        {
            get { return this._geoReplicationEnabled; }
            set { this._geoReplicationEnabled = value; }
        }
        
        private string _label;
        
        /// <summary>
        /// Required. A name for the storage account, specified as
        /// abase64-encoded string. The name may be up to 100 characters in
        /// length. The name can be used identify the storage account for your
        /// tracking purposes.
        /// </summary>
        public string Label
        {
            get
            {
                if (this._label == null)
                {
                    return this.Name;
                }
                else
                {
                    return this._label;
                }
            }
            set { this._label = value; }
        }
        
        private string _location;
        
        /// <summary>
        /// Optional. The location where the storage account is created.
        /// Required if AffinityGroup is not specified. You can include either
        /// a Location or AffinityGroup element in the request body, but not
        /// both. To list available locations, use the List Locations
        /// operation.
        /// </summary>
        public string Location
        {
            get { return this._location; }
            set { this._location = value; }
        }
        
        private string _name;
        
        /// <summary>
        /// Required. A name for the storage account, unique within Azure.
        /// Storage account names must be between 3 and 24 characters in
        /// length, and must use numbers and lower-case letters only. This
        /// name is the DNS prefix name and can be used to access blobs,
        /// queues, and tables in the storage account. For example:
        /// http://ServiceName.blob.core.windows.net/mycontainer/.
        /// </summary>
        public string Name
        {
            get { return this._name; }
            set { this._name = value; }
        }
        
        /// <summary>
        /// Initializes a new instance of the StorageAccountCreateParameters
        /// class.
        /// </summary>
        public StorageAccountCreateParameters()
        {
            this._extendedProperties = new Dictionary<string, string>();
        }
    }
}
