// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Xml.Linq;
using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.Common;
using Microsoft.WindowsAzure.Common.Internals;
using Microsoft.WindowsAzure.Management.WebSites;
using Microsoft.WindowsAzure.Management.WebSites.Models;

namespace Microsoft.WindowsAzure.Management.WebSites
{
    /// <summary>
    /// Operations for managing the web sites in a web space.  (see
    /// http://msdn.microsoft.com/en-us/library/windowsazure/dn166981.aspx for
    /// more information)
    /// </summary>
    internal partial class WebSiteOperations : IServiceOperations<WebSiteManagementClient>, Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations
    {
        /// <summary>
        /// Initializes a new instance of the WebSiteOperations class.
        /// </summary>
        /// <param name='client'>
        /// Reference to the service client.
        /// </param>
        internal WebSiteOperations(WebSiteManagementClient client)
        {
            this._client = client;
        }
        
        private WebSiteManagementClient _client;
        
        /// <summary>
        /// Gets a reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.WebSiteManagementClient.
        /// </summary>
        public WebSiteManagementClient Client
        {
            get { return this._client; }
        }
        
        /// <summary>
        /// You can swap a web site from one slot to the production slot.
        /// </summary>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <param name='slotName'>
        /// Required. The name of the web site slot to swap with the production
        /// slot.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified long-running
        /// operation, indicating whether it has succeeded, is inprogress, has
        /// timed out, or has failed. Note that this status is distinct from
        /// the HTTP status code returned for the Get Operation Status
        /// operation itself. If the long-running operation failed, the
        /// response body includes error information regarding the failure.
        /// </returns>
        public async System.Threading.Tasks.Task<Microsoft.WindowsAzure.Management.WebSites.Models.WebSiteOperationStatusResponse> BeginSwapingSlotsAsync(string webSpaceName, string webSiteName, string slotName, CancellationToken cancellationToken)
        {
            // Validate
            if (webSpaceName == null)
            {
                throw new ArgumentNullException("webSpaceName");
            }
            if (webSiteName == null)
            {
                throw new ArgumentNullException("webSiteName");
            }
            if (slotName == null)
            {
                throw new ArgumentNullException("slotName");
            }
            
            // Tracing
            bool shouldTrace = CloudContext.Configuration.Tracing.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = Tracing.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("webSpaceName", webSpaceName);
                tracingParameters.Add("webSiteName", webSiteName);
                tracingParameters.Add("slotName", slotName);
                Tracing.Enter(invocationId, this, "BeginSwapingSlotsAsync", tracingParameters);
            }
            
            // Construct URL
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            string url = "/" + this.Client.Credentials.SubscriptionId.Trim() + "/services/WebSpaces/" + webSpaceName.Trim() + "/sites/" + webSiteName.Trim() + "/slots?";
            url = url + "Command=swap";
            url = url + "&targetSlot=" + Uri.EscapeUriString(slotName.Trim());
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Post;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-version", "2013-08-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        Tracing.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        Tracing.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false), CloudExceptionType.Xml);
                        if (shouldTrace)
                        {
                            Tracing.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    WebSiteOperationStatusResponse result = null;
                    // Deserialize Response
                    cancellationToken.ThrowIfCancellationRequested();
                    string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                    result = new WebSiteOperationStatusResponse();
                    XDocument responseDoc = XDocument.Parse(responseContent);
                    
                    XElement operationElement = responseDoc.Element(XName.Get("Operation", "http://schemas.microsoft.com/windowsazure"));
                    if (operationElement != null && operationElement.IsEmpty == false)
                    {
                        XElement createdTimeElement = operationElement.Element(XName.Get("CreatedTime", "http://schemas.microsoft.com/windowsazure"));
                        if (createdTimeElement != null && createdTimeElement.IsEmpty == false)
                        {
                            DateTime createdTimeInstance = DateTime.Parse(createdTimeElement.Value, CultureInfo.InvariantCulture);
                            result.CreatedTime = createdTimeInstance;
                        }
                        
                        XElement errorsSequenceElement = operationElement.Element(XName.Get("Errors", "http://schemas.microsoft.com/windowsazure"));
                        if (errorsSequenceElement != null && errorsSequenceElement.IsEmpty == false)
                        {
                            bool isNil = false;
                            XAttribute nilAttribute = errorsSequenceElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                            if (nilAttribute != null)
                            {
                                isNil = nilAttribute.Value == "true";
                            }
                            if (isNil == false)
                            {
                                foreach (XElement errorsElement in errorsSequenceElement.Elements(XName.Get("Error", "http://schemas.microsoft.com/windowsazure")))
                                {
                                    WebSiteOperationStatusResponse.Error errorInstance = new WebSiteOperationStatusResponse.Error();
                                    result.Errors.Add(errorInstance);
                                    
                                    XElement codeElement = errorsElement.Element(XName.Get("Code", "http://schemas.microsoft.com/windowsazure"));
                                    if (codeElement != null && codeElement.IsEmpty == false)
                                    {
                                        bool isNil2 = false;
                                        XAttribute nilAttribute2 = codeElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                        if (nilAttribute2 != null)
                                        {
                                            isNil2 = nilAttribute2.Value == "true";
                                        }
                                        if (isNil2 == false)
                                        {
                                            string codeInstance = codeElement.Value;
                                            errorInstance.Code = codeInstance;
                                        }
                                    }
                                    
                                    XElement messageElement = errorsElement.Element(XName.Get("Message", "http://schemas.microsoft.com/windowsazure"));
                                    if (messageElement != null && messageElement.IsEmpty == false)
                                    {
                                        bool isNil3 = false;
                                        XAttribute nilAttribute3 = messageElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                        if (nilAttribute3 != null)
                                        {
                                            isNil3 = nilAttribute3.Value == "true";
                                        }
                                        if (isNil3 == false)
                                        {
                                            string messageInstance = messageElement.Value;
                                            errorInstance.Message = messageInstance;
                                        }
                                    }
                                    
                                    XElement extendedCodeElement = errorsElement.Element(XName.Get("ExtendedCode", "http://schemas.microsoft.com/windowsazure"));
                                    if (extendedCodeElement != null && extendedCodeElement.IsEmpty == false)
                                    {
                                        bool isNil4 = false;
                                        XAttribute nilAttribute4 = extendedCodeElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                        if (nilAttribute4 != null)
                                        {
                                            isNil4 = nilAttribute4.Value == "true";
                                        }
                                        if (isNil4 == false)
                                        {
                                            string extendedCodeInstance = extendedCodeElement.Value;
                                            errorInstance.ExtendedCode = extendedCodeInstance;
                                        }
                                    }
                                    
                                    XElement messageTemplateElement = errorsElement.Element(XName.Get("MessageTemplate", "http://schemas.microsoft.com/windowsazure"));
                                    if (messageTemplateElement != null && messageTemplateElement.IsEmpty == false)
                                    {
                                        bool isNil5 = false;
                                        XAttribute nilAttribute5 = messageTemplateElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                        if (nilAttribute5 != null)
                                        {
                                            isNil5 = nilAttribute5.Value == "true";
                                        }
                                        if (isNil5 == false)
                                        {
                                            string messageTemplateInstance = messageTemplateElement.Value;
                                            errorInstance.MessageTemplate = messageTemplateInstance;
                                        }
                                    }
                                    
                                    XElement parametersSequenceElement = errorsElement.Element(XName.Get("Parameters", "http://schemas.microsoft.com/windowsazure"));
                                    if (parametersSequenceElement != null && parametersSequenceElement.IsEmpty == false)
                                    {
                                        bool isNil6 = false;
                                        XAttribute nilAttribute6 = parametersSequenceElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                        if (nilAttribute6 != null)
                                        {
                                            isNil6 = nilAttribute6.Value == "true";
                                        }
                                        if (isNil6 == false)
                                        {
                                            foreach (XElement parametersElement in parametersSequenceElement.Elements(XName.Get("string", "http://schemas.microsoft.com/2003/10/Serialization/Arrays")))
                                            {
                                                errorInstance.Parameters.Add(parametersElement.Value);
                                            }
                                        }
                                    }
                                    
                                    XElement innerErrorsElement = errorsElement.Element(XName.Get("InnerErrors", "http://schemas.microsoft.com/windowsazure"));
                                    if (innerErrorsElement != null && innerErrorsElement.IsEmpty == false)
                                    {
                                        bool isNil7 = false;
                                        XAttribute nilAttribute7 = innerErrorsElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                        if (nilAttribute7 != null)
                                        {
                                            isNil7 = nilAttribute7.Value == "true";
                                        }
                                        if (isNil7 == false)
                                        {
                                            string innerErrorsInstance = innerErrorsElement.Value;
                                            errorInstance.InnerErrors = innerErrorsInstance;
                                        }
                                    }
                                }
                            }
                        }
                        
                        XElement expirationTimeElement = operationElement.Element(XName.Get("ExpirationTime", "http://schemas.microsoft.com/windowsazure"));
                        if (expirationTimeElement != null && expirationTimeElement.IsEmpty == false)
                        {
                            DateTime expirationTimeInstance = DateTime.Parse(expirationTimeElement.Value, CultureInfo.InvariantCulture);
                            result.ExpirationTime = expirationTimeInstance;
                        }
                        
                        XElement geoMasterOperationIdElement = operationElement.Element(XName.Get("GeoMasterOperationId", "http://schemas.microsoft.com/windowsazure"));
                        if (geoMasterOperationIdElement != null && geoMasterOperationIdElement.IsEmpty == false)
                        {
                            bool isNil8 = false;
                            XAttribute nilAttribute8 = geoMasterOperationIdElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                            if (nilAttribute8 != null)
                            {
                                isNil8 = nilAttribute8.Value == "true";
                            }
                            if (isNil8 == false)
                            {
                                string geoMasterOperationIdInstance = geoMasterOperationIdElement.Value;
                                result.GeoMasterOperationId = geoMasterOperationIdInstance;
                            }
                        }
                        
                        XElement idElement = operationElement.Element(XName.Get("Id", "http://schemas.microsoft.com/windowsazure"));
                        if (idElement != null && idElement.IsEmpty == false)
                        {
                            bool isNil9 = false;
                            XAttribute nilAttribute9 = idElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                            if (nilAttribute9 != null)
                            {
                                isNil9 = nilAttribute9.Value == "true";
                            }
                            if (isNil9 == false)
                            {
                                string idInstance = idElement.Value;
                                result.OperationId = idInstance;
                            }
                        }
                        
                        XElement modifiedTimeElement = operationElement.Element(XName.Get("ModifiedTime", "http://schemas.microsoft.com/windowsazure"));
                        if (modifiedTimeElement != null && modifiedTimeElement.IsEmpty == false)
                        {
                            DateTime modifiedTimeInstance = DateTime.Parse(modifiedTimeElement.Value, CultureInfo.InvariantCulture);
                            result.ModifiedTime = modifiedTimeInstance;
                        }
                        
                        XElement nameElement = operationElement.Element(XName.Get("Name", "http://schemas.microsoft.com/windowsazure"));
                        if (nameElement != null && nameElement.IsEmpty == false)
                        {
                            bool isNil10 = false;
                            XAttribute nilAttribute10 = nameElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                            if (nilAttribute10 != null)
                            {
                                isNil10 = nilAttribute10.Value == "true";
                            }
                            if (isNil10 == false)
                            {
                                string nameInstance = nameElement.Value;
                                result.Name = nameInstance;
                            }
                        }
                        
                        XElement statusElement = operationElement.Element(XName.Get("Status", "http://schemas.microsoft.com/windowsazure"));
                        if (statusElement != null && statusElement.IsEmpty == false)
                        {
                            WebSiteOperationStatus statusInstance = ((WebSiteOperationStatus)Enum.Parse(typeof(WebSiteOperationStatus), statusElement.Value, true));
                            result.Status = statusInstance;
                        }
                    }
                    
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        Tracing.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// You can create a web site by using a POST request that includes the
        /// name of the web site and other information in the request body.
        /// (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/dn166986.aspx
        /// for more information)
        /// </summary>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Create Web Site operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The Create Web Site operation response.
        /// </returns>
        public async System.Threading.Tasks.Task<Microsoft.WindowsAzure.Management.WebSites.Models.WebSiteCreateResponse> CreateAsync(string webSpaceName, WebSiteCreateParameters parameters, CancellationToken cancellationToken)
        {
            // Validate
            if (webSpaceName == null)
            {
                throw new ArgumentNullException("webSpaceName");
            }
            if (parameters == null)
            {
                throw new ArgumentNullException("parameters");
            }
            if (parameters.Name == null)
            {
                throw new ArgumentNullException("parameters.Name");
            }
            if (parameters.WebSpace != null)
            {
                if (parameters.WebSpace.GeoRegion == null)
                {
                    throw new ArgumentNullException("parameters.WebSpace.GeoRegion");
                }
                if (parameters.WebSpace.Name == null)
                {
                    throw new ArgumentNullException("parameters.WebSpace.Name");
                }
                if (parameters.WebSpace.Plan == null)
                {
                    throw new ArgumentNullException("parameters.WebSpace.Plan");
                }
            }
            if (parameters.WebSpaceName == null)
            {
                throw new ArgumentNullException("parameters.WebSpaceName");
            }
            
            // Tracing
            bool shouldTrace = CloudContext.Configuration.Tracing.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = Tracing.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("webSpaceName", webSpaceName);
                tracingParameters.Add("parameters", parameters);
                Tracing.Enter(invocationId, this, "CreateAsync", tracingParameters);
            }
            
            // Construct URL
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            string url = "/" + this.Client.Credentials.SubscriptionId.Trim() + "/services/WebSpaces/" + webSpaceName.Trim() + "/sites";
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Post;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-version", "2013-08-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Serialize Request
                string requestContent = null;
                XDocument requestDoc = new XDocument();
                
                XElement siteElement = new XElement(XName.Get("Site", "http://schemas.microsoft.com/windowsazure"));
                requestDoc.Add(siteElement);
                
                if (parameters.HostNames != null)
                {
                    XElement hostNamesSequenceElement = new XElement(XName.Get("HostNames", "http://schemas.microsoft.com/windowsazure"));
                    foreach (string hostNamesItem in parameters.HostNames)
                    {
                        XElement hostNamesItemElement = new XElement(XName.Get("string", "http://schemas.microsoft.com/2003/10/Serialization/Arrays"));
                        hostNamesItemElement.Value = hostNamesItem;
                        hostNamesSequenceElement.Add(hostNamesItemElement);
                    }
                    siteElement.Add(hostNamesSequenceElement);
                }
                
                XElement nameElement = new XElement(XName.Get("Name", "http://schemas.microsoft.com/windowsazure"));
                nameElement.Value = parameters.Name;
                siteElement.Add(nameElement);
                
                if (parameters.ComputeMode != null)
                {
                    XElement computeModeElement = new XElement(XName.Get("ComputeMode", "http://schemas.microsoft.com/windowsazure"));
                    computeModeElement.Value = parameters.ComputeMode.ToString();
                    siteElement.Add(computeModeElement);
                }
                
                if (parameters.ServerFarm != null)
                {
                    XElement serverFarmElement = new XElement(XName.Get("ServerFarm", "http://schemas.microsoft.com/windowsazure"));
                    serverFarmElement.Value = parameters.ServerFarm;
                    siteElement.Add(serverFarmElement);
                }
                
                if (parameters.SiteMode != null)
                {
                    XElement siteModeElement = new XElement(XName.Get("SiteMode", "http://schemas.microsoft.com/windowsazure"));
                    siteModeElement.Value = parameters.SiteMode.ToString();
                    siteElement.Add(siteModeElement);
                }
                
                XElement webSpaceElement = new XElement(XName.Get("WebSpace", "http://schemas.microsoft.com/windowsazure"));
                webSpaceElement.Value = parameters.WebSpaceName;
                siteElement.Add(webSpaceElement);
                
                if (parameters.WebSpace != null)
                {
                    XElement webSpaceToCreateElement = new XElement(XName.Get("WebSpaceToCreate", "http://schemas.microsoft.com/windowsazure"));
                    siteElement.Add(webSpaceToCreateElement);
                    
                    XElement geoRegionElement = new XElement(XName.Get("GeoRegion", "http://schemas.microsoft.com/windowsazure"));
                    geoRegionElement.Value = parameters.WebSpace.GeoRegion;
                    webSpaceToCreateElement.Add(geoRegionElement);
                    
                    XElement nameElement2 = new XElement(XName.Get("Name", "http://schemas.microsoft.com/windowsazure"));
                    nameElement2.Value = parameters.WebSpace.Name;
                    webSpaceToCreateElement.Add(nameElement2);
                    
                    XElement planElement = new XElement(XName.Get("Plan", "http://schemas.microsoft.com/windowsazure"));
                    planElement.Value = parameters.WebSpace.Plan;
                    webSpaceToCreateElement.Add(planElement);
                }
                
                requestContent = requestDoc.ToString();
                httpRequest.Content = new StringContent(requestContent, Encoding.UTF8);
                httpRequest.Content.Headers.ContentType = new MediaTypeHeaderValue("application/xml");
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        Tracing.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        Tracing.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Created)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, requestContent, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false), CloudExceptionType.Xml);
                        if (shouldTrace)
                        {
                            Tracing.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    WebSiteCreateResponse result = null;
                    // Deserialize Response
                    cancellationToken.ThrowIfCancellationRequested();
                    string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                    result = new WebSiteCreateResponse();
                    XDocument responseDoc = XDocument.Parse(responseContent);
                    
                    XElement siteElement2 = responseDoc.Element(XName.Get("Site", "http://schemas.microsoft.com/windowsazure"));
                    if (siteElement2 != null && siteElement2.IsEmpty == false)
                    {
                        WebSite webSiteInstance = new WebSite();
                        result.WebSite = webSiteInstance;
                        
                        XElement adminEnabledElement = siteElement2.Element(XName.Get("AdminEnabled", "http://schemas.microsoft.com/windowsazure"));
                        if (adminEnabledElement != null && adminEnabledElement.IsEmpty == false)
                        {
                            bool adminEnabledInstance = bool.Parse(adminEnabledElement.Value);
                            webSiteInstance.AdminEnabled = adminEnabledInstance;
                        }
                        
                        XElement availabilityStateElement = siteElement2.Element(XName.Get("AvailabilityState", "http://schemas.microsoft.com/windowsazure"));
                        if (availabilityStateElement != null && availabilityStateElement.IsEmpty == false)
                        {
                            WebSpaceAvailabilityState availabilityStateInstance = ((WebSpaceAvailabilityState)Enum.Parse(typeof(WebSpaceAvailabilityState), availabilityStateElement.Value, true));
                            webSiteInstance.AvailabilityState = availabilityStateInstance;
                        }
                        
                        XElement computeModeElement2 = siteElement2.Element(XName.Get("ComputeMode", "http://schemas.microsoft.com/windowsazure"));
                        if (computeModeElement2 != null && computeModeElement2.IsEmpty == false)
                        {
                            WebSiteComputeMode computeModeInstance = ((WebSiteComputeMode)Enum.Parse(typeof(WebSiteComputeMode), computeModeElement2.Value, true));
                            webSiteInstance.ComputeMode = computeModeInstance;
                        }
                        
                        XElement enabledElement = siteElement2.Element(XName.Get("Enabled", "http://schemas.microsoft.com/windowsazure"));
                        if (enabledElement != null && enabledElement.IsEmpty == false)
                        {
                            bool enabledInstance = bool.Parse(enabledElement.Value);
                            webSiteInstance.Enabled = enabledInstance;
                        }
                        
                        XElement enabledHostNamesSequenceElement = siteElement2.Element(XName.Get("EnabledHostNames", "http://schemas.microsoft.com/windowsazure"));
                        if (enabledHostNamesSequenceElement != null && enabledHostNamesSequenceElement.IsEmpty == false)
                        {
                            foreach (XElement enabledHostNamesElement in enabledHostNamesSequenceElement.Elements(XName.Get("string", "http://schemas.microsoft.com/2003/10/Serialization/Arrays")))
                            {
                                webSiteInstance.EnabledHostNames.Add(enabledHostNamesElement.Value);
                            }
                        }
                        
                        XElement hostNameSslStatesSequenceElement = siteElement2.Element(XName.Get("HostNameSslStates", "http://schemas.microsoft.com/windowsazure"));
                        if (hostNameSslStatesSequenceElement != null && hostNameSslStatesSequenceElement.IsEmpty == false)
                        {
                            foreach (XElement hostNameSslStatesElement in hostNameSslStatesSequenceElement.Elements(XName.Get("WebSiteHostNameSslState", "http://schemas.microsoft.com/windowsazure")))
                            {
                                WebSite.WebSiteHostNameSslState webSiteHostNameSslStateInstance = new WebSite.WebSiteHostNameSslState();
                                webSiteInstance.HostNameSslStates.Add(webSiteHostNameSslStateInstance);
                                
                                XElement nameElement3 = hostNameSslStatesElement.Element(XName.Get("Name", "http://schemas.microsoft.com/windowsazure"));
                                if (nameElement3 != null && nameElement3.IsEmpty == false)
                                {
                                    string nameInstance = nameElement3.Value;
                                    webSiteHostNameSslStateInstance.Name = nameInstance;
                                }
                                
                                XElement sslStateElement = hostNameSslStatesElement.Element(XName.Get("SslState", "http://schemas.microsoft.com/windowsazure"));
                                if (sslStateElement != null && sslStateElement.IsEmpty == false)
                                {
                                    WebSiteSslState sslStateInstance = ((WebSiteSslState)Enum.Parse(typeof(WebSiteSslState), sslStateElement.Value, true));
                                    webSiteHostNameSslStateInstance.SslState = sslStateInstance;
                                }
                                
                                XElement thumbprintElement = hostNameSslStatesElement.Element(XName.Get("Thumbprint", "http://schemas.microsoft.com/windowsazure"));
                                if (thumbprintElement != null && thumbprintElement.IsEmpty == false)
                                {
                                    bool isNil = false;
                                    XAttribute nilAttribute = thumbprintElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                    if (nilAttribute != null)
                                    {
                                        isNil = nilAttribute.Value == "true";
                                    }
                                    if (isNil == false)
                                    {
                                        string thumbprintInstance = thumbprintElement.Value;
                                        webSiteHostNameSslStateInstance.Thumbprint = thumbprintInstance;
                                    }
                                }
                                
                                XElement virtualIPElement = hostNameSslStatesElement.Element(XName.Get("VirtualIP", "http://schemas.microsoft.com/windowsazure"));
                                if (virtualIPElement != null && virtualIPElement.IsEmpty == false)
                                {
                                    bool isNil2 = false;
                                    XAttribute nilAttribute2 = virtualIPElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                    if (nilAttribute2 != null)
                                    {
                                        isNil2 = nilAttribute2.Value == "true";
                                    }
                                    if (isNil2 == false)
                                    {
                                        string virtualIPInstance = virtualIPElement.Value;
                                        webSiteHostNameSslStateInstance.VirtualIP = virtualIPInstance;
                                    }
                                }
                            }
                        }
                        
                        XElement hostNamesSequenceElement2 = siteElement2.Element(XName.Get("HostNames", "http://schemas.microsoft.com/windowsazure"));
                        if (hostNamesSequenceElement2 != null && hostNamesSequenceElement2.IsEmpty == false)
                        {
                            foreach (XElement hostNamesElement in hostNamesSequenceElement2.Elements(XName.Get("string", "http://schemas.microsoft.com/2003/10/Serialization/Arrays")))
                            {
                                webSiteInstance.HostNames.Add(hostNamesElement.Value);
                            }
                        }
                        
                        XElement lastModifiedTimeUtcElement = siteElement2.Element(XName.Get("LastModifiedTimeUtc", "http://schemas.microsoft.com/windowsazure"));
                        if (lastModifiedTimeUtcElement != null && lastModifiedTimeUtcElement.IsEmpty == false)
                        {
                            DateTime lastModifiedTimeUtcInstance = DateTime.Parse(lastModifiedTimeUtcElement.Value, CultureInfo.InvariantCulture);
                            webSiteInstance.LastModifiedTimeUtc = lastModifiedTimeUtcInstance;
                        }
                        
                        XElement nameElement4 = siteElement2.Element(XName.Get("Name", "http://schemas.microsoft.com/windowsazure"));
                        if (nameElement4 != null && nameElement4.IsEmpty == false)
                        {
                            string nameInstance2 = nameElement4.Value;
                            webSiteInstance.Name = nameInstance2;
                        }
                        
                        XElement ownerElement = siteElement2.Element(XName.Get("Owner", "http://schemas.microsoft.com/windowsazure"));
                        if (ownerElement != null && ownerElement.IsEmpty == false)
                        {
                            bool isNil3 = false;
                            XAttribute nilAttribute3 = ownerElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                            if (nilAttribute3 != null)
                            {
                                isNil3 = nilAttribute3.Value == "true";
                            }
                            if (isNil3 == false)
                            {
                                string ownerInstance = ownerElement.Value;
                                webSiteInstance.Owner = ownerInstance;
                            }
                        }
                        
                        XElement repositorySiteNameElement = siteElement2.Element(XName.Get("RepositorySiteName", "http://schemas.microsoft.com/windowsazure"));
                        if (repositorySiteNameElement != null && repositorySiteNameElement.IsEmpty == false)
                        {
                            string repositorySiteNameInstance = repositorySiteNameElement.Value;
                            webSiteInstance.RepositorySiteName = repositorySiteNameInstance;
                        }
                        
                        XElement runtimeAvailabilityStateElement = siteElement2.Element(XName.Get("RuntimeAvailabilityState", "http://schemas.microsoft.com/windowsazure"));
                        if (runtimeAvailabilityStateElement != null && runtimeAvailabilityStateElement.IsEmpty == false)
                        {
                            WebSiteRuntimeAvailabilityState runtimeAvailabilityStateInstance = ((WebSiteRuntimeAvailabilityState)Enum.Parse(typeof(WebSiteRuntimeAvailabilityState), runtimeAvailabilityStateElement.Value, true));
                            webSiteInstance.RuntimeAvailabilityState = runtimeAvailabilityStateInstance;
                        }
                        
                        XElement sSLCertificatesSequenceElement = siteElement2.Element(XName.Get("SSLCertificates", "http://schemas.microsoft.com/windowsazure"));
                        if (sSLCertificatesSequenceElement != null && sSLCertificatesSequenceElement.IsEmpty == false)
                        {
                            bool isNil4 = false;
                            XAttribute nilAttribute4 = sSLCertificatesSequenceElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                            if (nilAttribute4 != null)
                            {
                                isNil4 = nilAttribute4.Value == "true";
                            }
                            if (isNil4 == false)
                            {
                                foreach (XElement sSLCertificatesElement in sSLCertificatesSequenceElement.Elements(XName.Get("Certificate", "http://schemas.microsoft.com/windowsazure")))
                                {
                                    WebSite.WebSiteSslCertificate certificateInstance = new WebSite.WebSiteSslCertificate();
                                    webSiteInstance.SslCertificates.Add(certificateInstance);
                                    
                                    XElement expirationDateElement = sSLCertificatesElement.Element(XName.Get("ExpirationDate", "http://schemas.microsoft.com/windowsazure"));
                                    if (expirationDateElement != null && expirationDateElement.IsEmpty == false && string.IsNullOrEmpty(expirationDateElement.Value) == false)
                                    {
                                        bool isNil5 = false;
                                        XAttribute nilAttribute5 = expirationDateElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                        if (nilAttribute5 != null)
                                        {
                                            isNil5 = nilAttribute5.Value == "true";
                                        }
                                        if (isNil5 == false)
                                        {
                                            DateTime expirationDateInstance = DateTime.Parse(expirationDateElement.Value, CultureInfo.InvariantCulture);
                                            certificateInstance.ExpirationDate = expirationDateInstance;
                                        }
                                    }
                                    
                                    XElement friendlyNameElement = sSLCertificatesElement.Element(XName.Get("FriendlyName", "http://schemas.microsoft.com/windowsazure"));
                                    if (friendlyNameElement != null && friendlyNameElement.IsEmpty == false)
                                    {
                                        bool isNil6 = false;
                                        XAttribute nilAttribute6 = friendlyNameElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                        if (nilAttribute6 != null)
                                        {
                                            isNil6 = nilAttribute6.Value == "true";
                                        }
                                        if (isNil6 == false)
                                        {
                                            string friendlyNameInstance = friendlyNameElement.Value;
                                            certificateInstance.FriendlyName = friendlyNameInstance;
                                        }
                                    }
                                    
                                    XElement hostNamesSequenceElement3 = sSLCertificatesElement.Element(XName.Get("HostNames", "http://schemas.microsoft.com/windowsazure"));
                                    if (hostNamesSequenceElement3 != null && hostNamesSequenceElement3.IsEmpty == false)
                                    {
                                        bool isNil7 = false;
                                        XAttribute nilAttribute7 = hostNamesSequenceElement3.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                        if (nilAttribute7 != null)
                                        {
                                            isNil7 = nilAttribute7.Value == "true";
                                        }
                                        if (isNil7 == false)
                                        {
                                            foreach (XElement hostNamesElement2 in hostNamesSequenceElement3.Elements(XName.Get("string", "http://schemas.microsoft.com/2003/10/Serialization/Arrays")))
                                            {
                                                certificateInstance.HostNames.Add(hostNamesElement2.Value);
                                            }
                                        }
                                    }
                                    
                                    XElement issueDateElement = sSLCertificatesElement.Element(XName.Get("IssueDate", "http://schemas.microsoft.com/windowsazure"));
                                    if (issueDateElement != null && issueDateElement.IsEmpty == false && string.IsNullOrEmpty(issueDateElement.Value) == false)
                                    {
                                        bool isNil8 = false;
                                        XAttribute nilAttribute8 = issueDateElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                        if (nilAttribute8 != null)
                                        {
                                            isNil8 = nilAttribute8.Value == "true";
                                        }
                                        if (isNil8 == false)
                                        {
                                            DateTime issueDateInstance = DateTime.Parse(issueDateElement.Value, CultureInfo.InvariantCulture);
                                            certificateInstance.IssueDate = issueDateInstance;
                                        }
                                    }
                                    
                                    XElement issuerElement = sSLCertificatesElement.Element(XName.Get("Issuer", "http://schemas.microsoft.com/windowsazure"));
                                    if (issuerElement != null && issuerElement.IsEmpty == false)
                                    {
                                        bool isNil9 = false;
                                        XAttribute nilAttribute9 = issuerElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                        if (nilAttribute9 != null)
                                        {
                                            isNil9 = nilAttribute9.Value == "true";
                                        }
                                        if (isNil9 == false)
                                        {
                                            string issuerInstance = issuerElement.Value;
                                            certificateInstance.Issuer = issuerInstance;
                                        }
                                    }
                                    
                                    XElement passwordElement = sSLCertificatesElement.Element(XName.Get("Password", "http://schemas.microsoft.com/windowsazure"));
                                    if (passwordElement != null && passwordElement.IsEmpty == false)
                                    {
                                        bool isNil10 = false;
                                        XAttribute nilAttribute10 = passwordElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                        if (nilAttribute10 != null)
                                        {
                                            isNil10 = nilAttribute10.Value == "true";
                                        }
                                        if (isNil10 == false)
                                        {
                                            string passwordInstance = passwordElement.Value;
                                            certificateInstance.Password = passwordInstance;
                                        }
                                    }
                                    
                                    XElement pfxBlobElement = sSLCertificatesElement.Element(XName.Get("PfxBlob", "http://schemas.microsoft.com/windowsazure"));
                                    if (pfxBlobElement != null && pfxBlobElement.IsEmpty == false)
                                    {
                                        bool isNil11 = false;
                                        XAttribute nilAttribute11 = pfxBlobElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                        if (nilAttribute11 != null)
                                        {
                                            isNil11 = nilAttribute11.Value == "true";
                                        }
                                        if (isNil11 == false)
                                        {
                                            byte[] pfxBlobInstance = Convert.FromBase64String(pfxBlobElement.Value);
                                            certificateInstance.PfxBlob = pfxBlobInstance;
                                        }
                                    }
                                    
                                    XElement selfLinkElement = sSLCertificatesElement.Element(XName.Get("SelfLink", "http://schemas.microsoft.com/windowsazure"));
                                    if (selfLinkElement != null && selfLinkElement.IsEmpty == false)
                                    {
                                        bool isNil12 = false;
                                        XAttribute nilAttribute12 = selfLinkElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                        if (nilAttribute12 != null)
                                        {
                                            isNil12 = nilAttribute12.Value == "true";
                                        }
                                        if (isNil12 == false)
                                        {
                                            Uri selfLinkInstance = TypeConversion.TryParseUri(selfLinkElement.Value);
                                            certificateInstance.SelfLinkUri = selfLinkInstance;
                                        }
                                    }
                                    
                                    XElement siteNameElement = sSLCertificatesElement.Element(XName.Get("SiteName", "http://schemas.microsoft.com/windowsazure"));
                                    if (siteNameElement != null && siteNameElement.IsEmpty == false)
                                    {
                                        bool isNil13 = false;
                                        XAttribute nilAttribute13 = siteNameElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                        if (nilAttribute13 != null)
                                        {
                                            isNil13 = nilAttribute13.Value == "true";
                                        }
                                        if (isNil13 == false)
                                        {
                                            string siteNameInstance = siteNameElement.Value;
                                            certificateInstance.SiteName = siteNameInstance;
                                        }
                                    }
                                    
                                    XElement subjectNameElement = sSLCertificatesElement.Element(XName.Get("SubjectName", "http://schemas.microsoft.com/windowsazure"));
                                    if (subjectNameElement != null && subjectNameElement.IsEmpty == false)
                                    {
                                        bool isNil14 = false;
                                        XAttribute nilAttribute14 = subjectNameElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                        if (nilAttribute14 != null)
                                        {
                                            isNil14 = nilAttribute14.Value == "true";
                                        }
                                        if (isNil14 == false)
                                        {
                                            string subjectNameInstance = subjectNameElement.Value;
                                            certificateInstance.SubjectName = subjectNameInstance;
                                        }
                                    }
                                    
                                    XElement thumbprintElement2 = sSLCertificatesElement.Element(XName.Get("Thumbprint", "http://schemas.microsoft.com/windowsazure"));
                                    if (thumbprintElement2 != null && thumbprintElement2.IsEmpty == false)
                                    {
                                        bool isNil15 = false;
                                        XAttribute nilAttribute15 = thumbprintElement2.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                        if (nilAttribute15 != null)
                                        {
                                            isNil15 = nilAttribute15.Value == "true";
                                        }
                                        if (isNil15 == false)
                                        {
                                            string thumbprintInstance2 = thumbprintElement2.Value;
                                            certificateInstance.Thumbprint = thumbprintInstance2;
                                        }
                                    }
                                    
                                    XElement toDeleteElement = sSLCertificatesElement.Element(XName.Get("ToDelete", "http://schemas.microsoft.com/windowsazure"));
                                    if (toDeleteElement != null && toDeleteElement.IsEmpty == false && string.IsNullOrEmpty(toDeleteElement.Value) == false)
                                    {
                                        bool isNil16 = false;
                                        XAttribute nilAttribute16 = toDeleteElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                        if (nilAttribute16 != null)
                                        {
                                            isNil16 = nilAttribute16.Value == "true";
                                        }
                                        if (isNil16 == false)
                                        {
                                            bool toDeleteInstance = bool.Parse(toDeleteElement.Value);
                                            certificateInstance.IsToBeDeleted = toDeleteInstance;
                                        }
                                    }
                                    
                                    XElement validElement = sSLCertificatesElement.Element(XName.Get("Valid", "http://schemas.microsoft.com/windowsazure"));
                                    if (validElement != null && validElement.IsEmpty == false && string.IsNullOrEmpty(validElement.Value) == false)
                                    {
                                        bool isNil17 = false;
                                        XAttribute nilAttribute17 = validElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                        if (nilAttribute17 != null)
                                        {
                                            isNil17 = nilAttribute17.Value == "true";
                                        }
                                        if (isNil17 == false)
                                        {
                                            bool validInstance = bool.Parse(validElement.Value);
                                            certificateInstance.IsValid = validInstance;
                                        }
                                    }
                                }
                            }
                        }
                        
                        XElement selfLinkElement2 = siteElement2.Element(XName.Get("SelfLink", "http://schemas.microsoft.com/windowsazure"));
                        if (selfLinkElement2 != null && selfLinkElement2.IsEmpty == false)
                        {
                            Uri selfLinkInstance2 = TypeConversion.TryParseUri(selfLinkElement2.Value);
                            webSiteInstance.Uri = selfLinkInstance2;
                        }
                        
                        XElement serverFarmElement2 = siteElement2.Element(XName.Get("ServerFarm", "http://schemas.microsoft.com/windowsazure"));
                        if (serverFarmElement2 != null && serverFarmElement2.IsEmpty == false)
                        {
                            string serverFarmInstance = serverFarmElement2.Value;
                            webSiteInstance.ServerFarm = serverFarmInstance;
                        }
                        
                        XElement siteModeElement2 = siteElement2.Element(XName.Get("SiteMode", "http://schemas.microsoft.com/windowsazure"));
                        if (siteModeElement2 != null && siteModeElement2.IsEmpty == false)
                        {
                            WebSiteMode siteModeInstance = ((WebSiteMode)Enum.Parse(typeof(WebSiteMode), siteModeElement2.Value, true));
                            webSiteInstance.SiteMode = siteModeInstance;
                        }
                        
                        XElement sitePropertiesElement = siteElement2.Element(XName.Get("SiteProperties", "http://schemas.microsoft.com/windowsazure"));
                        if (sitePropertiesElement != null && sitePropertiesElement.IsEmpty == false)
                        {
                            WebSite.WebSiteProperties sitePropertiesInstance = new WebSite.WebSiteProperties();
                            webSiteInstance.SiteProperties = sitePropertiesInstance;
                            
                            XElement appSettingsSequenceElement = sitePropertiesElement.Element(XName.Get("AppSettings", "http://schemas.microsoft.com/windowsazure"));
                            if (appSettingsSequenceElement != null && appSettingsSequenceElement.IsEmpty == false)
                            {
                                foreach (XElement appSettingsElement in appSettingsSequenceElement.Elements(XName.Get("NameValuePair", "http://schemas.microsoft.com/windowsazure")))
                                {
                                    string appSettingsKey = appSettingsElement.Element(XName.Get("Name", "http://schemas.microsoft.com/windowsazure")).Value;
                                    string appSettingsValue = appSettingsElement.Element(XName.Get("Value", "http://schemas.microsoft.com/windowsazure")).Value;
                                    sitePropertiesInstance.AppSettings.Add(appSettingsKey, appSettingsValue);
                                }
                            }
                            
                            XElement metadataSequenceElement = sitePropertiesElement.Element(XName.Get("Metadata", "http://schemas.microsoft.com/windowsazure"));
                            if (metadataSequenceElement != null && metadataSequenceElement.IsEmpty == false)
                            {
                                foreach (XElement metadataElement in metadataSequenceElement.Elements(XName.Get("NameValuePair", "http://schemas.microsoft.com/windowsazure")))
                                {
                                    string metadataKey = metadataElement.Element(XName.Get("Name", "http://schemas.microsoft.com/windowsazure")).Value;
                                    string metadataValue = metadataElement.Element(XName.Get("Value", "http://schemas.microsoft.com/windowsazure")).Value;
                                    sitePropertiesInstance.Metadata.Add(metadataKey, metadataValue);
                                }
                            }
                            
                            XElement propertiesSequenceElement = sitePropertiesElement.Element(XName.Get("Properties", "http://schemas.microsoft.com/windowsazure"));
                            if (propertiesSequenceElement != null && propertiesSequenceElement.IsEmpty == false)
                            {
                                foreach (XElement propertiesElement in propertiesSequenceElement.Elements(XName.Get("NameValuePair", "http://schemas.microsoft.com/windowsazure")))
                                {
                                    string propertiesKey = propertiesElement.Element(XName.Get("Name", "http://schemas.microsoft.com/windowsazure")).Value;
                                    string propertiesValue = propertiesElement.Element(XName.Get("Value", "http://schemas.microsoft.com/windowsazure")).Value;
                                    sitePropertiesInstance.Properties.Add(propertiesKey, propertiesValue);
                                }
                            }
                        }
                        
                        XElement stateElement = siteElement2.Element(XName.Get("State", "http://schemas.microsoft.com/windowsazure"));
                        if (stateElement != null && stateElement.IsEmpty == false)
                        {
                            string stateInstance = stateElement.Value;
                            webSiteInstance.State = stateInstance;
                        }
                        
                        XElement usageStateElement = siteElement2.Element(XName.Get("UsageState", "http://schemas.microsoft.com/windowsazure"));
                        if (usageStateElement != null && usageStateElement.IsEmpty == false)
                        {
                            WebSiteUsageState usageStateInstance = ((WebSiteUsageState)Enum.Parse(typeof(WebSiteUsageState), usageStateElement.Value, true));
                            webSiteInstance.UsageState = usageStateInstance;
                        }
                        
                        XElement webSpaceElement2 = siteElement2.Element(XName.Get("WebSpace", "http://schemas.microsoft.com/windowsazure"));
                        if (webSpaceElement2 != null && webSpaceElement2.IsEmpty == false)
                        {
                            string webSpaceInstance = webSpaceElement2.Value;
                            webSiteInstance.WebSpace = webSpaceInstance;
                        }
                    }
                    
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        Tracing.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// A web site repository is essentially a Git repository that you can
        /// use to manage your web site content. By using Git source control
        /// tools, you can push or pull version-controlled changes to your
        /// site. This API executes a repository create operation.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/dn166967.aspx
        /// for more information)
        /// </summary>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public async System.Threading.Tasks.Task<OperationResponse> CreateRepositoryAsync(string webSpaceName, string webSiteName, CancellationToken cancellationToken)
        {
            // Validate
            if (webSpaceName == null)
            {
                throw new ArgumentNullException("webSpaceName");
            }
            if (webSiteName == null)
            {
                throw new ArgumentNullException("webSiteName");
            }
            
            // Tracing
            bool shouldTrace = CloudContext.Configuration.Tracing.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = Tracing.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("webSpaceName", webSpaceName);
                tracingParameters.Add("webSiteName", webSiteName);
                Tracing.Enter(invocationId, this, "CreateRepositoryAsync", tracingParameters);
            }
            
            // Construct URL
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            string url = "/" + this.Client.Credentials.SubscriptionId.Trim() + "/services/WebSpaces/" + webSpaceName.Trim() + "/sites/" + webSiteName.Trim() + "/repository";
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Post;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-version", "2013-08-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        Tracing.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        Tracing.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false), CloudExceptionType.Xml);
                        if (shouldTrace)
                        {
                            Tracing.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    OperationResponse result = null;
                    result = new OperationResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        Tracing.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// You can delete a web site by issuing an HTTP DELETE request. If the
        /// web site being deleted is the only site remaining in a server
        /// farm, you can optionally delete the server farm as well by using
        /// the deleteEmptyServerFarm parameter.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/dn236430.aspx
        /// for more information)
        /// </summary>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Delete Web Site operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public async System.Threading.Tasks.Task<OperationResponse> DeleteAsync(string webSpaceName, string webSiteName, WebSiteDeleteParameters parameters, CancellationToken cancellationToken)
        {
            // Validate
            if (webSpaceName == null)
            {
                throw new ArgumentNullException("webSpaceName");
            }
            if (webSiteName == null)
            {
                throw new ArgumentNullException("webSiteName");
            }
            if (parameters == null)
            {
                throw new ArgumentNullException("parameters");
            }
            
            // Tracing
            bool shouldTrace = CloudContext.Configuration.Tracing.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = Tracing.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("webSpaceName", webSpaceName);
                tracingParameters.Add("webSiteName", webSiteName);
                tracingParameters.Add("parameters", parameters);
                Tracing.Enter(invocationId, this, "DeleteAsync", tracingParameters);
            }
            
            // Construct URL
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            string url = "/" + this.Client.Credentials.SubscriptionId.Trim() + "/services/WebSpaces/" + webSpaceName.Trim() + "/sites/" + webSiteName.Trim() + "?";
            url = url + "deleteEmptyServerFarm=" + Uri.EscapeUriString(parameters.DeleteEmptyServerFarm.ToString().ToLower());
            url = url + "&deleteMetrics=" + Uri.EscapeUriString(parameters.DeleteMetrics.ToString().ToLower());
            url = url + "&deleteAllSlots=" + Uri.EscapeUriString(parameters.DeleteAllSlots.ToString().ToLower());
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Delete;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-version", "2013-08-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        Tracing.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        Tracing.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false), CloudExceptionType.Xml);
                        if (shouldTrace)
                        {
                            Tracing.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    OperationResponse result = null;
                    result = new OperationResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        Tracing.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// A web site repository is essentially a Git repository that you can
        /// use to manage your web site content. By using Git source control
        /// tools, you can push or pull version-controlled changes to your
        /// site. This API executes a repository delete operation.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/dn166967.aspx
        /// for more information)
        /// </summary>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The Delete Repository Web Site operation response.
        /// </returns>
        public async System.Threading.Tasks.Task<Microsoft.WindowsAzure.Management.WebSites.Models.WebSiteDeleteRepositoryResponse> DeleteRepositoryAsync(string webSpaceName, string webSiteName, CancellationToken cancellationToken)
        {
            // Validate
            if (webSpaceName == null)
            {
                throw new ArgumentNullException("webSpaceName");
            }
            if (webSiteName == null)
            {
                throw new ArgumentNullException("webSiteName");
            }
            
            // Tracing
            bool shouldTrace = CloudContext.Configuration.Tracing.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = Tracing.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("webSpaceName", webSpaceName);
                tracingParameters.Add("webSiteName", webSiteName);
                Tracing.Enter(invocationId, this, "DeleteRepositoryAsync", tracingParameters);
            }
            
            // Construct URL
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            string url = "/" + this.Client.Credentials.SubscriptionId.Trim() + "/services/WebSpaces/" + webSpaceName.Trim() + "/sites/" + webSiteName.Trim() + "/repository";
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Delete;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-version", "2013-08-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        Tracing.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        Tracing.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false), CloudExceptionType.Xml);
                        if (shouldTrace)
                        {
                            Tracing.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    WebSiteDeleteRepositoryResponse result = null;
                    // Deserialize Response
                    cancellationToken.ThrowIfCancellationRequested();
                    string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                    result = new WebSiteDeleteRepositoryResponse();
                    XDocument responseDoc = XDocument.Parse(responseContent);
                    
                    XElement anyURIElement = responseDoc.Element(XName.Get("anyURI", "http://schemas.microsoft.com/2003/10/Serialization/"));
                    if (anyURIElement != null && anyURIElement.IsEmpty == false)
                    {
                        result.Uri = TypeConversion.TryParseUri(anyURIElement.Value);
                    }
                    
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        Tracing.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// You can generate a new random password for publishing a site by
        /// issuing an HTTP POST request. Tip: If you want to verify that the
        /// publish password has changed, issue an HTTP GET on /publishxml
        /// before calling /newpassword. In the publish XML, note the hash
        /// value in the userPWD attribute. After calling /newpassword, call
        /// /publishxml again. You can then compare the new value of userPWD
        /// in the Publish XML with the one you noted earlier.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/dn236428.aspx
        /// for more information)
        /// </summary>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public async System.Threading.Tasks.Task<OperationResponse> GeneratePasswordAsync(string webSpaceName, string webSiteName, CancellationToken cancellationToken)
        {
            // Validate
            if (webSpaceName == null)
            {
                throw new ArgumentNullException("webSpaceName");
            }
            if (webSiteName == null)
            {
                throw new ArgumentNullException("webSiteName");
            }
            
            // Tracing
            bool shouldTrace = CloudContext.Configuration.Tracing.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = Tracing.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("webSpaceName", webSpaceName);
                tracingParameters.Add("webSiteName", webSiteName);
                Tracing.Enter(invocationId, this, "GeneratePasswordAsync", tracingParameters);
            }
            
            // Construct URL
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            string url = "/" + this.Client.Credentials.SubscriptionId.Trim() + "/services/WebSpaces/" + webSpaceName.Trim() + "/sites/" + webSiteName.Trim() + "/newpassword";
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Post;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-version", "2013-08-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        Tracing.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        Tracing.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false), CloudExceptionType.Xml);
                        if (shouldTrace)
                        {
                            Tracing.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    OperationResponse result = null;
                    result = new OperationResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        Tracing.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// You can retrieve details for a web site by issuing an HTTP GET
        /// request.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/dn167007.aspx
        /// for more information)
        /// </summary>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <param name='parameters'>
        /// Optional. Parameters supplied to the Get Web Site Operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The Get Web Site operation response.
        /// </returns>
        public async System.Threading.Tasks.Task<Microsoft.WindowsAzure.Management.WebSites.Models.WebSiteGetResponse> GetAsync(string webSpaceName, string webSiteName, WebSiteGetParameters parameters, CancellationToken cancellationToken)
        {
            // Validate
            if (webSpaceName == null)
            {
                throw new ArgumentNullException("webSpaceName");
            }
            if (webSiteName == null)
            {
                throw new ArgumentNullException("webSiteName");
            }
            
            // Tracing
            bool shouldTrace = CloudContext.Configuration.Tracing.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = Tracing.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("webSpaceName", webSpaceName);
                tracingParameters.Add("webSiteName", webSiteName);
                tracingParameters.Add("parameters", parameters);
                Tracing.Enter(invocationId, this, "GetAsync", tracingParameters);
            }
            
            // Construct URL
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            string url = "/" + this.Client.Credentials.SubscriptionId.Trim() + "/services/WebSpaces/" + webSpaceName.Trim() + "/sites/" + webSiteName.Trim() + "?";
            if (parameters != null && parameters.PropertiesToInclude != null && parameters.PropertiesToInclude.Count > 0)
            {
                url = url + "propertiesToInclude=" + Uri.EscapeUriString(string.Join(",", parameters.PropertiesToInclude));
            }
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-version", "2013-08-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        Tracing.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        Tracing.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false), CloudExceptionType.Xml);
                        if (shouldTrace)
                        {
                            Tracing.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    WebSiteGetResponse result = null;
                    // Deserialize Response
                    cancellationToken.ThrowIfCancellationRequested();
                    string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                    result = new WebSiteGetResponse();
                    XDocument responseDoc = XDocument.Parse(responseContent);
                    
                    XElement siteElement = responseDoc.Element(XName.Get("Site", "http://schemas.microsoft.com/windowsazure"));
                    if (siteElement != null && siteElement.IsEmpty == false)
                    {
                        WebSite webSiteInstance = new WebSite();
                        result.WebSite = webSiteInstance;
                        
                        XElement adminEnabledElement = siteElement.Element(XName.Get("AdminEnabled", "http://schemas.microsoft.com/windowsazure"));
                        if (adminEnabledElement != null && adminEnabledElement.IsEmpty == false)
                        {
                            bool adminEnabledInstance = bool.Parse(adminEnabledElement.Value);
                            webSiteInstance.AdminEnabled = adminEnabledInstance;
                        }
                        
                        XElement availabilityStateElement = siteElement.Element(XName.Get("AvailabilityState", "http://schemas.microsoft.com/windowsazure"));
                        if (availabilityStateElement != null && availabilityStateElement.IsEmpty == false)
                        {
                            WebSpaceAvailabilityState availabilityStateInstance = ((WebSpaceAvailabilityState)Enum.Parse(typeof(WebSpaceAvailabilityState), availabilityStateElement.Value, true));
                            webSiteInstance.AvailabilityState = availabilityStateInstance;
                        }
                        
                        XElement computeModeElement = siteElement.Element(XName.Get("ComputeMode", "http://schemas.microsoft.com/windowsazure"));
                        if (computeModeElement != null && computeModeElement.IsEmpty == false)
                        {
                            WebSiteComputeMode computeModeInstance = ((WebSiteComputeMode)Enum.Parse(typeof(WebSiteComputeMode), computeModeElement.Value, true));
                            webSiteInstance.ComputeMode = computeModeInstance;
                        }
                        
                        XElement enabledElement = siteElement.Element(XName.Get("Enabled", "http://schemas.microsoft.com/windowsazure"));
                        if (enabledElement != null && enabledElement.IsEmpty == false)
                        {
                            bool enabledInstance = bool.Parse(enabledElement.Value);
                            webSiteInstance.Enabled = enabledInstance;
                        }
                        
                        XElement enabledHostNamesSequenceElement = siteElement.Element(XName.Get("EnabledHostNames", "http://schemas.microsoft.com/windowsazure"));
                        if (enabledHostNamesSequenceElement != null && enabledHostNamesSequenceElement.IsEmpty == false)
                        {
                            foreach (XElement enabledHostNamesElement in enabledHostNamesSequenceElement.Elements(XName.Get("string", "http://schemas.microsoft.com/2003/10/Serialization/Arrays")))
                            {
                                webSiteInstance.EnabledHostNames.Add(enabledHostNamesElement.Value);
                            }
                        }
                        
                        XElement hostNameSslStatesSequenceElement = siteElement.Element(XName.Get("HostNameSslStates", "http://schemas.microsoft.com/windowsazure"));
                        if (hostNameSslStatesSequenceElement != null && hostNameSslStatesSequenceElement.IsEmpty == false)
                        {
                            foreach (XElement hostNameSslStatesElement in hostNameSslStatesSequenceElement.Elements(XName.Get("WebSiteHostNameSslState", "http://schemas.microsoft.com/windowsazure")))
                            {
                                WebSite.WebSiteHostNameSslState webSiteHostNameSslStateInstance = new WebSite.WebSiteHostNameSslState();
                                webSiteInstance.HostNameSslStates.Add(webSiteHostNameSslStateInstance);
                                
                                XElement nameElement = hostNameSslStatesElement.Element(XName.Get("Name", "http://schemas.microsoft.com/windowsazure"));
                                if (nameElement != null && nameElement.IsEmpty == false)
                                {
                                    string nameInstance = nameElement.Value;
                                    webSiteHostNameSslStateInstance.Name = nameInstance;
                                }
                                
                                XElement sslStateElement = hostNameSslStatesElement.Element(XName.Get("SslState", "http://schemas.microsoft.com/windowsazure"));
                                if (sslStateElement != null && sslStateElement.IsEmpty == false)
                                {
                                    WebSiteSslState sslStateInstance = ((WebSiteSslState)Enum.Parse(typeof(WebSiteSslState), sslStateElement.Value, true));
                                    webSiteHostNameSslStateInstance.SslState = sslStateInstance;
                                }
                                
                                XElement thumbprintElement = hostNameSslStatesElement.Element(XName.Get("Thumbprint", "http://schemas.microsoft.com/windowsazure"));
                                if (thumbprintElement != null && thumbprintElement.IsEmpty == false)
                                {
                                    bool isNil = false;
                                    XAttribute nilAttribute = thumbprintElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                    if (nilAttribute != null)
                                    {
                                        isNil = nilAttribute.Value == "true";
                                    }
                                    if (isNil == false)
                                    {
                                        string thumbprintInstance = thumbprintElement.Value;
                                        webSiteHostNameSslStateInstance.Thumbprint = thumbprintInstance;
                                    }
                                }
                                
                                XElement virtualIPElement = hostNameSslStatesElement.Element(XName.Get("VirtualIP", "http://schemas.microsoft.com/windowsazure"));
                                if (virtualIPElement != null && virtualIPElement.IsEmpty == false)
                                {
                                    bool isNil2 = false;
                                    XAttribute nilAttribute2 = virtualIPElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                    if (nilAttribute2 != null)
                                    {
                                        isNil2 = nilAttribute2.Value == "true";
                                    }
                                    if (isNil2 == false)
                                    {
                                        string virtualIPInstance = virtualIPElement.Value;
                                        webSiteHostNameSslStateInstance.VirtualIP = virtualIPInstance;
                                    }
                                }
                            }
                        }
                        
                        XElement hostNamesSequenceElement = siteElement.Element(XName.Get("HostNames", "http://schemas.microsoft.com/windowsazure"));
                        if (hostNamesSequenceElement != null && hostNamesSequenceElement.IsEmpty == false)
                        {
                            foreach (XElement hostNamesElement in hostNamesSequenceElement.Elements(XName.Get("string", "http://schemas.microsoft.com/2003/10/Serialization/Arrays")))
                            {
                                webSiteInstance.HostNames.Add(hostNamesElement.Value);
                            }
                        }
                        
                        XElement lastModifiedTimeUtcElement = siteElement.Element(XName.Get("LastModifiedTimeUtc", "http://schemas.microsoft.com/windowsazure"));
                        if (lastModifiedTimeUtcElement != null && lastModifiedTimeUtcElement.IsEmpty == false)
                        {
                            DateTime lastModifiedTimeUtcInstance = DateTime.Parse(lastModifiedTimeUtcElement.Value, CultureInfo.InvariantCulture);
                            webSiteInstance.LastModifiedTimeUtc = lastModifiedTimeUtcInstance;
                        }
                        
                        XElement nameElement2 = siteElement.Element(XName.Get("Name", "http://schemas.microsoft.com/windowsazure"));
                        if (nameElement2 != null && nameElement2.IsEmpty == false)
                        {
                            string nameInstance2 = nameElement2.Value;
                            webSiteInstance.Name = nameInstance2;
                        }
                        
                        XElement ownerElement = siteElement.Element(XName.Get("Owner", "http://schemas.microsoft.com/windowsazure"));
                        if (ownerElement != null && ownerElement.IsEmpty == false)
                        {
                            bool isNil3 = false;
                            XAttribute nilAttribute3 = ownerElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                            if (nilAttribute3 != null)
                            {
                                isNil3 = nilAttribute3.Value == "true";
                            }
                            if (isNil3 == false)
                            {
                                string ownerInstance = ownerElement.Value;
                                webSiteInstance.Owner = ownerInstance;
                            }
                        }
                        
                        XElement repositorySiteNameElement = siteElement.Element(XName.Get("RepositorySiteName", "http://schemas.microsoft.com/windowsazure"));
                        if (repositorySiteNameElement != null && repositorySiteNameElement.IsEmpty == false)
                        {
                            string repositorySiteNameInstance = repositorySiteNameElement.Value;
                            webSiteInstance.RepositorySiteName = repositorySiteNameInstance;
                        }
                        
                        XElement runtimeAvailabilityStateElement = siteElement.Element(XName.Get("RuntimeAvailabilityState", "http://schemas.microsoft.com/windowsazure"));
                        if (runtimeAvailabilityStateElement != null && runtimeAvailabilityStateElement.IsEmpty == false)
                        {
                            WebSiteRuntimeAvailabilityState runtimeAvailabilityStateInstance = ((WebSiteRuntimeAvailabilityState)Enum.Parse(typeof(WebSiteRuntimeAvailabilityState), runtimeAvailabilityStateElement.Value, true));
                            webSiteInstance.RuntimeAvailabilityState = runtimeAvailabilityStateInstance;
                        }
                        
                        XElement sSLCertificatesSequenceElement = siteElement.Element(XName.Get("SSLCertificates", "http://schemas.microsoft.com/windowsazure"));
                        if (sSLCertificatesSequenceElement != null && sSLCertificatesSequenceElement.IsEmpty == false)
                        {
                            bool isNil4 = false;
                            XAttribute nilAttribute4 = sSLCertificatesSequenceElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                            if (nilAttribute4 != null)
                            {
                                isNil4 = nilAttribute4.Value == "true";
                            }
                            if (isNil4 == false)
                            {
                                foreach (XElement sSLCertificatesElement in sSLCertificatesSequenceElement.Elements(XName.Get("Certificate", "http://schemas.microsoft.com/windowsazure")))
                                {
                                    WebSite.WebSiteSslCertificate certificateInstance = new WebSite.WebSiteSslCertificate();
                                    webSiteInstance.SslCertificates.Add(certificateInstance);
                                    
                                    XElement expirationDateElement = sSLCertificatesElement.Element(XName.Get("ExpirationDate", "http://schemas.microsoft.com/windowsazure"));
                                    if (expirationDateElement != null && expirationDateElement.IsEmpty == false && string.IsNullOrEmpty(expirationDateElement.Value) == false)
                                    {
                                        bool isNil5 = false;
                                        XAttribute nilAttribute5 = expirationDateElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                        if (nilAttribute5 != null)
                                        {
                                            isNil5 = nilAttribute5.Value == "true";
                                        }
                                        if (isNil5 == false)
                                        {
                                            DateTime expirationDateInstance = DateTime.Parse(expirationDateElement.Value, CultureInfo.InvariantCulture);
                                            certificateInstance.ExpirationDate = expirationDateInstance;
                                        }
                                    }
                                    
                                    XElement friendlyNameElement = sSLCertificatesElement.Element(XName.Get("FriendlyName", "http://schemas.microsoft.com/windowsazure"));
                                    if (friendlyNameElement != null && friendlyNameElement.IsEmpty == false)
                                    {
                                        bool isNil6 = false;
                                        XAttribute nilAttribute6 = friendlyNameElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                        if (nilAttribute6 != null)
                                        {
                                            isNil6 = nilAttribute6.Value == "true";
                                        }
                                        if (isNil6 == false)
                                        {
                                            string friendlyNameInstance = friendlyNameElement.Value;
                                            certificateInstance.FriendlyName = friendlyNameInstance;
                                        }
                                    }
                                    
                                    XElement hostNamesSequenceElement2 = sSLCertificatesElement.Element(XName.Get("HostNames", "http://schemas.microsoft.com/windowsazure"));
                                    if (hostNamesSequenceElement2 != null && hostNamesSequenceElement2.IsEmpty == false)
                                    {
                                        bool isNil7 = false;
                                        XAttribute nilAttribute7 = hostNamesSequenceElement2.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                        if (nilAttribute7 != null)
                                        {
                                            isNil7 = nilAttribute7.Value == "true";
                                        }
                                        if (isNil7 == false)
                                        {
                                            foreach (XElement hostNamesElement2 in hostNamesSequenceElement2.Elements(XName.Get("string", "http://schemas.microsoft.com/2003/10/Serialization/Arrays")))
                                            {
                                                certificateInstance.HostNames.Add(hostNamesElement2.Value);
                                            }
                                        }
                                    }
                                    
                                    XElement issueDateElement = sSLCertificatesElement.Element(XName.Get("IssueDate", "http://schemas.microsoft.com/windowsazure"));
                                    if (issueDateElement != null && issueDateElement.IsEmpty == false && string.IsNullOrEmpty(issueDateElement.Value) == false)
                                    {
                                        bool isNil8 = false;
                                        XAttribute nilAttribute8 = issueDateElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                        if (nilAttribute8 != null)
                                        {
                                            isNil8 = nilAttribute8.Value == "true";
                                        }
                                        if (isNil8 == false)
                                        {
                                            DateTime issueDateInstance = DateTime.Parse(issueDateElement.Value, CultureInfo.InvariantCulture);
                                            certificateInstance.IssueDate = issueDateInstance;
                                        }
                                    }
                                    
                                    XElement issuerElement = sSLCertificatesElement.Element(XName.Get("Issuer", "http://schemas.microsoft.com/windowsazure"));
                                    if (issuerElement != null && issuerElement.IsEmpty == false)
                                    {
                                        bool isNil9 = false;
                                        XAttribute nilAttribute9 = issuerElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                        if (nilAttribute9 != null)
                                        {
                                            isNil9 = nilAttribute9.Value == "true";
                                        }
                                        if (isNil9 == false)
                                        {
                                            string issuerInstance = issuerElement.Value;
                                            certificateInstance.Issuer = issuerInstance;
                                        }
                                    }
                                    
                                    XElement passwordElement = sSLCertificatesElement.Element(XName.Get("Password", "http://schemas.microsoft.com/windowsazure"));
                                    if (passwordElement != null && passwordElement.IsEmpty == false)
                                    {
                                        bool isNil10 = false;
                                        XAttribute nilAttribute10 = passwordElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                        if (nilAttribute10 != null)
                                        {
                                            isNil10 = nilAttribute10.Value == "true";
                                        }
                                        if (isNil10 == false)
                                        {
                                            string passwordInstance = passwordElement.Value;
                                            certificateInstance.Password = passwordInstance;
                                        }
                                    }
                                    
                                    XElement pfxBlobElement = sSLCertificatesElement.Element(XName.Get("PfxBlob", "http://schemas.microsoft.com/windowsazure"));
                                    if (pfxBlobElement != null && pfxBlobElement.IsEmpty == false)
                                    {
                                        bool isNil11 = false;
                                        XAttribute nilAttribute11 = pfxBlobElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                        if (nilAttribute11 != null)
                                        {
                                            isNil11 = nilAttribute11.Value == "true";
                                        }
                                        if (isNil11 == false)
                                        {
                                            byte[] pfxBlobInstance = Convert.FromBase64String(pfxBlobElement.Value);
                                            certificateInstance.PfxBlob = pfxBlobInstance;
                                        }
                                    }
                                    
                                    XElement selfLinkElement = sSLCertificatesElement.Element(XName.Get("SelfLink", "http://schemas.microsoft.com/windowsazure"));
                                    if (selfLinkElement != null && selfLinkElement.IsEmpty == false)
                                    {
                                        bool isNil12 = false;
                                        XAttribute nilAttribute12 = selfLinkElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                        if (nilAttribute12 != null)
                                        {
                                            isNil12 = nilAttribute12.Value == "true";
                                        }
                                        if (isNil12 == false)
                                        {
                                            Uri selfLinkInstance = TypeConversion.TryParseUri(selfLinkElement.Value);
                                            certificateInstance.SelfLinkUri = selfLinkInstance;
                                        }
                                    }
                                    
                                    XElement siteNameElement = sSLCertificatesElement.Element(XName.Get("SiteName", "http://schemas.microsoft.com/windowsazure"));
                                    if (siteNameElement != null && siteNameElement.IsEmpty == false)
                                    {
                                        bool isNil13 = false;
                                        XAttribute nilAttribute13 = siteNameElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                        if (nilAttribute13 != null)
                                        {
                                            isNil13 = nilAttribute13.Value == "true";
                                        }
                                        if (isNil13 == false)
                                        {
                                            string siteNameInstance = siteNameElement.Value;
                                            certificateInstance.SiteName = siteNameInstance;
                                        }
                                    }
                                    
                                    XElement subjectNameElement = sSLCertificatesElement.Element(XName.Get("SubjectName", "http://schemas.microsoft.com/windowsazure"));
                                    if (subjectNameElement != null && subjectNameElement.IsEmpty == false)
                                    {
                                        bool isNil14 = false;
                                        XAttribute nilAttribute14 = subjectNameElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                        if (nilAttribute14 != null)
                                        {
                                            isNil14 = nilAttribute14.Value == "true";
                                        }
                                        if (isNil14 == false)
                                        {
                                            string subjectNameInstance = subjectNameElement.Value;
                                            certificateInstance.SubjectName = subjectNameInstance;
                                        }
                                    }
                                    
                                    XElement thumbprintElement2 = sSLCertificatesElement.Element(XName.Get("Thumbprint", "http://schemas.microsoft.com/windowsazure"));
                                    if (thumbprintElement2 != null && thumbprintElement2.IsEmpty == false)
                                    {
                                        bool isNil15 = false;
                                        XAttribute nilAttribute15 = thumbprintElement2.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                        if (nilAttribute15 != null)
                                        {
                                            isNil15 = nilAttribute15.Value == "true";
                                        }
                                        if (isNil15 == false)
                                        {
                                            string thumbprintInstance2 = thumbprintElement2.Value;
                                            certificateInstance.Thumbprint = thumbprintInstance2;
                                        }
                                    }
                                    
                                    XElement toDeleteElement = sSLCertificatesElement.Element(XName.Get("ToDelete", "http://schemas.microsoft.com/windowsazure"));
                                    if (toDeleteElement != null && toDeleteElement.IsEmpty == false && string.IsNullOrEmpty(toDeleteElement.Value) == false)
                                    {
                                        bool isNil16 = false;
                                        XAttribute nilAttribute16 = toDeleteElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                        if (nilAttribute16 != null)
                                        {
                                            isNil16 = nilAttribute16.Value == "true";
                                        }
                                        if (isNil16 == false)
                                        {
                                            bool toDeleteInstance = bool.Parse(toDeleteElement.Value);
                                            certificateInstance.IsToBeDeleted = toDeleteInstance;
                                        }
                                    }
                                    
                                    XElement validElement = sSLCertificatesElement.Element(XName.Get("Valid", "http://schemas.microsoft.com/windowsazure"));
                                    if (validElement != null && validElement.IsEmpty == false && string.IsNullOrEmpty(validElement.Value) == false)
                                    {
                                        bool isNil17 = false;
                                        XAttribute nilAttribute17 = validElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                        if (nilAttribute17 != null)
                                        {
                                            isNil17 = nilAttribute17.Value == "true";
                                        }
                                        if (isNil17 == false)
                                        {
                                            bool validInstance = bool.Parse(validElement.Value);
                                            certificateInstance.IsValid = validInstance;
                                        }
                                    }
                                }
                            }
                        }
                        
                        XElement selfLinkElement2 = siteElement.Element(XName.Get("SelfLink", "http://schemas.microsoft.com/windowsazure"));
                        if (selfLinkElement2 != null && selfLinkElement2.IsEmpty == false)
                        {
                            Uri selfLinkInstance2 = TypeConversion.TryParseUri(selfLinkElement2.Value);
                            webSiteInstance.Uri = selfLinkInstance2;
                        }
                        
                        XElement serverFarmElement = siteElement.Element(XName.Get("ServerFarm", "http://schemas.microsoft.com/windowsazure"));
                        if (serverFarmElement != null && serverFarmElement.IsEmpty == false)
                        {
                            string serverFarmInstance = serverFarmElement.Value;
                            webSiteInstance.ServerFarm = serverFarmInstance;
                        }
                        
                        XElement siteModeElement = siteElement.Element(XName.Get("SiteMode", "http://schemas.microsoft.com/windowsazure"));
                        if (siteModeElement != null && siteModeElement.IsEmpty == false)
                        {
                            WebSiteMode siteModeInstance = ((WebSiteMode)Enum.Parse(typeof(WebSiteMode), siteModeElement.Value, true));
                            webSiteInstance.SiteMode = siteModeInstance;
                        }
                        
                        XElement sitePropertiesElement = siteElement.Element(XName.Get("SiteProperties", "http://schemas.microsoft.com/windowsazure"));
                        if (sitePropertiesElement != null && sitePropertiesElement.IsEmpty == false)
                        {
                            WebSite.WebSiteProperties sitePropertiesInstance = new WebSite.WebSiteProperties();
                            webSiteInstance.SiteProperties = sitePropertiesInstance;
                            
                            XElement appSettingsSequenceElement = sitePropertiesElement.Element(XName.Get("AppSettings", "http://schemas.microsoft.com/windowsazure"));
                            if (appSettingsSequenceElement != null && appSettingsSequenceElement.IsEmpty == false)
                            {
                                foreach (XElement appSettingsElement in appSettingsSequenceElement.Elements(XName.Get("NameValuePair", "http://schemas.microsoft.com/windowsazure")))
                                {
                                    string appSettingsKey = appSettingsElement.Element(XName.Get("Name", "http://schemas.microsoft.com/windowsazure")).Value;
                                    string appSettingsValue = appSettingsElement.Element(XName.Get("Value", "http://schemas.microsoft.com/windowsazure")).Value;
                                    sitePropertiesInstance.AppSettings.Add(appSettingsKey, appSettingsValue);
                                }
                            }
                            
                            XElement metadataSequenceElement = sitePropertiesElement.Element(XName.Get("Metadata", "http://schemas.microsoft.com/windowsazure"));
                            if (metadataSequenceElement != null && metadataSequenceElement.IsEmpty == false)
                            {
                                foreach (XElement metadataElement in metadataSequenceElement.Elements(XName.Get("NameValuePair", "http://schemas.microsoft.com/windowsazure")))
                                {
                                    string metadataKey = metadataElement.Element(XName.Get("Name", "http://schemas.microsoft.com/windowsazure")).Value;
                                    string metadataValue = metadataElement.Element(XName.Get("Value", "http://schemas.microsoft.com/windowsazure")).Value;
                                    sitePropertiesInstance.Metadata.Add(metadataKey, metadataValue);
                                }
                            }
                            
                            XElement propertiesSequenceElement = sitePropertiesElement.Element(XName.Get("Properties", "http://schemas.microsoft.com/windowsazure"));
                            if (propertiesSequenceElement != null && propertiesSequenceElement.IsEmpty == false)
                            {
                                foreach (XElement propertiesElement in propertiesSequenceElement.Elements(XName.Get("NameValuePair", "http://schemas.microsoft.com/windowsazure")))
                                {
                                    string propertiesKey = propertiesElement.Element(XName.Get("Name", "http://schemas.microsoft.com/windowsazure")).Value;
                                    string propertiesValue = propertiesElement.Element(XName.Get("Value", "http://schemas.microsoft.com/windowsazure")).Value;
                                    sitePropertiesInstance.Properties.Add(propertiesKey, propertiesValue);
                                }
                            }
                        }
                        
                        XElement stateElement = siteElement.Element(XName.Get("State", "http://schemas.microsoft.com/windowsazure"));
                        if (stateElement != null && stateElement.IsEmpty == false)
                        {
                            string stateInstance = stateElement.Value;
                            webSiteInstance.State = stateInstance;
                        }
                        
                        XElement usageStateElement = siteElement.Element(XName.Get("UsageState", "http://schemas.microsoft.com/windowsazure"));
                        if (usageStateElement != null && usageStateElement.IsEmpty == false)
                        {
                            WebSiteUsageState usageStateInstance = ((WebSiteUsageState)Enum.Parse(typeof(WebSiteUsageState), usageStateElement.Value, true));
                            webSiteInstance.UsageState = usageStateInstance;
                        }
                        
                        XElement webSpaceElement = siteElement.Element(XName.Get("WebSpace", "http://schemas.microsoft.com/windowsazure"));
                        if (webSpaceElement != null && webSpaceElement.IsEmpty == false)
                        {
                            string webSpaceInstance = webSpaceElement.Value;
                            webSiteInstance.WebSpace = webSpaceInstance;
                        }
                    }
                    
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        Tracing.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// You can retrieve the config settings for a web site by issuing an
        /// HTTP GET request.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/dn166985.aspx
        /// for more information)
        /// </summary>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The Get Configuration Web Site operation response.
        /// </returns>
        public async System.Threading.Tasks.Task<Microsoft.WindowsAzure.Management.WebSites.Models.WebSiteGetConfigurationResponse> GetConfigurationAsync(string webSpaceName, string webSiteName, CancellationToken cancellationToken)
        {
            // Validate
            if (webSpaceName == null)
            {
                throw new ArgumentNullException("webSpaceName");
            }
            if (webSiteName == null)
            {
                throw new ArgumentNullException("webSiteName");
            }
            
            // Tracing
            bool shouldTrace = CloudContext.Configuration.Tracing.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = Tracing.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("webSpaceName", webSpaceName);
                tracingParameters.Add("webSiteName", webSiteName);
                Tracing.Enter(invocationId, this, "GetConfigurationAsync", tracingParameters);
            }
            
            // Construct URL
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            string url = "/" + this.Client.Credentials.SubscriptionId.Trim() + "/services/WebSpaces/" + webSpaceName.Trim() + "/sites/" + webSiteName.Trim() + "/config";
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-version", "2013-08-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        Tracing.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        Tracing.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false), CloudExceptionType.Xml);
                        if (shouldTrace)
                        {
                            Tracing.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    WebSiteGetConfigurationResponse result = null;
                    // Deserialize Response
                    cancellationToken.ThrowIfCancellationRequested();
                    string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                    result = new WebSiteGetConfigurationResponse();
                    XDocument responseDoc = XDocument.Parse(responseContent);
                    
                    XElement siteConfigElement = responseDoc.Element(XName.Get("SiteConfig", "http://schemas.microsoft.com/windowsazure"));
                    if (siteConfigElement != null && siteConfigElement.IsEmpty == false)
                    {
                        XElement appSettingsSequenceElement = siteConfigElement.Element(XName.Get("AppSettings", "http://schemas.microsoft.com/windowsazure"));
                        if (appSettingsSequenceElement != null && appSettingsSequenceElement.IsEmpty == false)
                        {
                            foreach (XElement appSettingsElement in appSettingsSequenceElement.Elements(XName.Get("NameValuePair", "http://schemas.microsoft.com/windowsazure")))
                            {
                                string appSettingsKey = appSettingsElement.Element(XName.Get("Name", "http://schemas.microsoft.com/windowsazure")).Value;
                                string appSettingsValue = appSettingsElement.Element(XName.Get("Value", "http://schemas.microsoft.com/windowsazure")).Value;
                                result.AppSettings.Add(appSettingsKey, appSettingsValue);
                            }
                        }
                        
                        XElement connectionStringsSequenceElement = siteConfigElement.Element(XName.Get("ConnectionStrings", "http://schemas.microsoft.com/windowsazure"));
                        if (connectionStringsSequenceElement != null && connectionStringsSequenceElement.IsEmpty == false)
                        {
                            foreach (XElement connectionStringsElement in connectionStringsSequenceElement.Elements(XName.Get("ConnStringInfo", "http://schemas.microsoft.com/windowsazure")))
                            {
                                WebSiteGetConfigurationResponse.ConnectionStringInfo connStringInfoInstance = new WebSiteGetConfigurationResponse.ConnectionStringInfo();
                                result.ConnectionStrings.Add(connStringInfoInstance);
                                
                                XElement connectionStringElement = connectionStringsElement.Element(XName.Get("ConnectionString", "http://schemas.microsoft.com/windowsazure"));
                                if (connectionStringElement != null && connectionStringElement.IsEmpty == false)
                                {
                                    string connectionStringInstance = connectionStringElement.Value;
                                    connStringInfoInstance.ConnectionString = connectionStringInstance;
                                }
                                
                                XElement nameElement = connectionStringsElement.Element(XName.Get("Name", "http://schemas.microsoft.com/windowsazure"));
                                if (nameElement != null && nameElement.IsEmpty == false)
                                {
                                    string nameInstance = nameElement.Value;
                                    connStringInfoInstance.Name = nameInstance;
                                }
                                
                                XElement typeElement = connectionStringsElement.Element(XName.Get("Type", "http://schemas.microsoft.com/windowsazure"));
                                if (typeElement != null && typeElement.IsEmpty == false)
                                {
                                    string typeInstance = typeElement.Value;
                                    connStringInfoInstance.Type = typeInstance;
                                }
                            }
                        }
                        
                        XElement defaultDocumentsSequenceElement = siteConfigElement.Element(XName.Get("DefaultDocuments", "http://schemas.microsoft.com/windowsazure"));
                        if (defaultDocumentsSequenceElement != null && defaultDocumentsSequenceElement.IsEmpty == false)
                        {
                            foreach (XElement defaultDocumentsElement in defaultDocumentsSequenceElement.Elements(XName.Get("string", "http://schemas.microsoft.com/2003/10/Serialization/Arrays")))
                            {
                                result.DefaultDocuments.Add(defaultDocumentsElement.Value);
                            }
                        }
                        
                        XElement detailedErrorLoggingEnabledElement = siteConfigElement.Element(XName.Get("DetailedErrorLoggingEnabled", "http://schemas.microsoft.com/windowsazure"));
                        if (detailedErrorLoggingEnabledElement != null && detailedErrorLoggingEnabledElement.IsEmpty == false && string.IsNullOrEmpty(detailedErrorLoggingEnabledElement.Value) == false)
                        {
                            bool detailedErrorLoggingEnabledInstance = bool.Parse(detailedErrorLoggingEnabledElement.Value);
                            result.DetailedErrorLoggingEnabled = detailedErrorLoggingEnabledInstance;
                        }
                        
                        XElement documentRootElement = siteConfigElement.Element(XName.Get("DocumentRoot", "http://schemas.microsoft.com/windowsazure"));
                        if (documentRootElement != null && documentRootElement.IsEmpty == false)
                        {
                            string documentRootInstance = documentRootElement.Value;
                            result.DocumentRoot = documentRootInstance;
                        }
                        
                        XElement handlerMappingsSequenceElement = siteConfigElement.Element(XName.Get("HandlerMappings", "http://schemas.microsoft.com/windowsazure"));
                        if (handlerMappingsSequenceElement != null && handlerMappingsSequenceElement.IsEmpty == false)
                        {
                            foreach (XElement handlerMappingsElement in handlerMappingsSequenceElement.Elements(XName.Get("HandlerMapping", "http://schemas.microsoft.com/windowsazure")))
                            {
                                WebSiteGetConfigurationResponse.HandlerMapping handlerMappingInstance = new WebSiteGetConfigurationResponse.HandlerMapping();
                                result.HandlerMappings.Add(handlerMappingInstance);
                                
                                XElement argumentsElement = handlerMappingsElement.Element(XName.Get("Arguments", "http://schemas.microsoft.com/windowsazure"));
                                if (argumentsElement != null && argumentsElement.IsEmpty == false)
                                {
                                    string argumentsInstance = argumentsElement.Value;
                                    handlerMappingInstance.Arguments = argumentsInstance;
                                }
                                
                                XElement extensionElement = handlerMappingsElement.Element(XName.Get("Extension", "http://schemas.microsoft.com/windowsazure"));
                                if (extensionElement != null && extensionElement.IsEmpty == false)
                                {
                                    string extensionInstance = extensionElement.Value;
                                    handlerMappingInstance.Extension = extensionInstance;
                                }
                                
                                XElement scriptProcessorElement = handlerMappingsElement.Element(XName.Get("ScriptProcessor", "http://schemas.microsoft.com/windowsazure"));
                                if (scriptProcessorElement != null && scriptProcessorElement.IsEmpty == false)
                                {
                                    string scriptProcessorInstance = scriptProcessorElement.Value;
                                    handlerMappingInstance.ScriptProcessor = scriptProcessorInstance;
                                }
                            }
                        }
                        
                        XElement httpLoggingEnabledElement = siteConfigElement.Element(XName.Get("HttpLoggingEnabled", "http://schemas.microsoft.com/windowsazure"));
                        if (httpLoggingEnabledElement != null && httpLoggingEnabledElement.IsEmpty == false && string.IsNullOrEmpty(httpLoggingEnabledElement.Value) == false)
                        {
                            bool httpLoggingEnabledInstance = bool.Parse(httpLoggingEnabledElement.Value);
                            result.HttpLoggingEnabled = httpLoggingEnabledInstance;
                        }
                        
                        XElement logsDirectorySizeLimitElement = siteConfigElement.Element(XName.Get("LogsDirectorySizeLimit", "http://schemas.microsoft.com/windowsazure"));
                        if (logsDirectorySizeLimitElement != null && logsDirectorySizeLimitElement.IsEmpty == false && string.IsNullOrEmpty(logsDirectorySizeLimitElement.Value) == false)
                        {
                            int logsDirectorySizeLimitInstance = int.Parse(logsDirectorySizeLimitElement.Value, CultureInfo.InvariantCulture);
                            result.LogsDirectorySizeLimit = logsDirectorySizeLimitInstance;
                        }
                        
                        XElement managedPipelineModeElement = siteConfigElement.Element(XName.Get("ManagedPipelineMode", "http://schemas.microsoft.com/windowsazure"));
                        if (managedPipelineModeElement != null && managedPipelineModeElement.IsEmpty == false && string.IsNullOrEmpty(managedPipelineModeElement.Value) == false)
                        {
                            ManagedPipelineMode managedPipelineModeInstance = ((ManagedPipelineMode)Enum.Parse(typeof(ManagedPipelineMode), managedPipelineModeElement.Value, true));
                            result.ManagedPipelineMode = managedPipelineModeInstance;
                        }
                        
                        XElement metadataSequenceElement = siteConfigElement.Element(XName.Get("Metadata", "http://schemas.microsoft.com/windowsazure"));
                        if (metadataSequenceElement != null && metadataSequenceElement.IsEmpty == false)
                        {
                            foreach (XElement metadataElement in metadataSequenceElement.Elements(XName.Get("NameValuePair", "http://schemas.microsoft.com/windowsazure")))
                            {
                                string metadataKey = metadataElement.Element(XName.Get("Name", "http://schemas.microsoft.com/windowsazure")).Value;
                                string metadataValue = metadataElement.Element(XName.Get("Value", "http://schemas.microsoft.com/windowsazure")).Value;
                                result.Metadata.Add(metadataKey, metadataValue);
                            }
                        }
                        
                        XElement netFrameworkVersionElement = siteConfigElement.Element(XName.Get("NetFrameworkVersion", "http://schemas.microsoft.com/windowsazure"));
                        if (netFrameworkVersionElement != null && netFrameworkVersionElement.IsEmpty == false)
                        {
                            string netFrameworkVersionInstance = netFrameworkVersionElement.Value;
                            result.NetFrameworkVersion = netFrameworkVersionInstance;
                        }
                        
                        XElement numberOfWorkersElement = siteConfigElement.Element(XName.Get("NumberOfWorkers", "http://schemas.microsoft.com/windowsazure"));
                        if (numberOfWorkersElement != null && numberOfWorkersElement.IsEmpty == false && string.IsNullOrEmpty(numberOfWorkersElement.Value) == false)
                        {
                            int numberOfWorkersInstance = int.Parse(numberOfWorkersElement.Value, CultureInfo.InvariantCulture);
                            result.NumberOfWorkers = numberOfWorkersInstance;
                        }
                        
                        XElement phpVersionElement = siteConfigElement.Element(XName.Get("PhpVersion", "http://schemas.microsoft.com/windowsazure"));
                        if (phpVersionElement != null && phpVersionElement.IsEmpty == false)
                        {
                            string phpVersionInstance = phpVersionElement.Value;
                            result.PhpVersion = phpVersionInstance;
                        }
                        
                        XElement publishingPasswordElement = siteConfigElement.Element(XName.Get("PublishingPassword", "http://schemas.microsoft.com/windowsazure"));
                        if (publishingPasswordElement != null && publishingPasswordElement.IsEmpty == false)
                        {
                            string publishingPasswordInstance = publishingPasswordElement.Value;
                            result.PublishingPassword = publishingPasswordInstance;
                        }
                        
                        XElement publishingUsernameElement = siteConfigElement.Element(XName.Get("PublishingUsername", "http://schemas.microsoft.com/windowsazure"));
                        if (publishingUsernameElement != null && publishingUsernameElement.IsEmpty == false)
                        {
                            string publishingUsernameInstance = publishingUsernameElement.Value;
                            result.PublishingUserName = publishingUsernameInstance;
                        }
                        
                        XElement remoteDebuggingEnabledElement = siteConfigElement.Element(XName.Get("RemoteDebuggingEnabled", "http://schemas.microsoft.com/windowsazure"));
                        if (remoteDebuggingEnabledElement != null && remoteDebuggingEnabledElement.IsEmpty == false && string.IsNullOrEmpty(remoteDebuggingEnabledElement.Value) == false)
                        {
                            bool remoteDebuggingEnabledInstance = bool.Parse(remoteDebuggingEnabledElement.Value);
                            result.RemoteDebuggingEnabled = remoteDebuggingEnabledInstance;
                        }
                        
                        XElement remoteDebuggingVersionElement = siteConfigElement.Element(XName.Get("RemoteDebuggingVersion", "http://schemas.microsoft.com/windowsazure"));
                        if (remoteDebuggingVersionElement != null && remoteDebuggingVersionElement.IsEmpty == false)
                        {
                            bool isNil = false;
                            XAttribute nilAttribute = remoteDebuggingVersionElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                            if (nilAttribute != null)
                            {
                                isNil = nilAttribute.Value == "true";
                            }
                            if (isNil == false)
                            {
                                RemoteDebuggingVersion remoteDebuggingVersionInstance = ((RemoteDebuggingVersion)Enum.Parse(typeof(RemoteDebuggingVersion), remoteDebuggingVersionElement.Value, true));
                                result.RemoteDebuggingVersion = remoteDebuggingVersionInstance;
                            }
                        }
                        
                        XElement requestTracingEnabledElement = siteConfigElement.Element(XName.Get("RequestTracingEnabled", "http://schemas.microsoft.com/windowsazure"));
                        if (requestTracingEnabledElement != null && requestTracingEnabledElement.IsEmpty == false && string.IsNullOrEmpty(requestTracingEnabledElement.Value) == false)
                        {
                            bool requestTracingEnabledInstance = bool.Parse(requestTracingEnabledElement.Value);
                            result.RequestTracingEnabled = requestTracingEnabledInstance;
                        }
                        
                        XElement requestTracingExpirationTimeElement = siteConfigElement.Element(XName.Get("RequestTracingExpirationTime", "http://schemas.microsoft.com/windowsazure"));
                        if (requestTracingExpirationTimeElement != null && requestTracingExpirationTimeElement.IsEmpty == false && string.IsNullOrEmpty(requestTracingExpirationTimeElement.Value) == false)
                        {
                            bool isNil2 = false;
                            XAttribute nilAttribute2 = requestTracingExpirationTimeElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                            if (nilAttribute2 != null)
                            {
                                isNil2 = nilAttribute2.Value == "true";
                            }
                            if (isNil2 == false)
                            {
                                DateTime requestTracingExpirationTimeInstance = DateTime.Parse(requestTracingExpirationTimeElement.Value, CultureInfo.InvariantCulture);
                                result.RequestTracingExpirationTime = requestTracingExpirationTimeInstance;
                            }
                        }
                        
                        XElement scmTypeElement = siteConfigElement.Element(XName.Get("ScmType", "http://schemas.microsoft.com/windowsazure"));
                        if (scmTypeElement != null && scmTypeElement.IsEmpty == false)
                        {
                            string scmTypeInstance = scmTypeElement.Value;
                            result.ScmType = scmTypeInstance;
                        }
                        
                        XElement use32BitWorkerProcessElement = siteConfigElement.Element(XName.Get("Use32BitWorkerProcess", "http://schemas.microsoft.com/windowsazure"));
                        if (use32BitWorkerProcessElement != null && use32BitWorkerProcessElement.IsEmpty == false && string.IsNullOrEmpty(use32BitWorkerProcessElement.Value) == false)
                        {
                            bool use32BitWorkerProcessInstance = bool.Parse(use32BitWorkerProcessElement.Value);
                            result.Use32BitWorkerProcess = use32BitWorkerProcessInstance;
                        }
                        
                        XElement webSocketsEnabledElement = siteConfigElement.Element(XName.Get("WebSocketsEnabled", "http://schemas.microsoft.com/windowsazure"));
                        if (webSocketsEnabledElement != null && webSocketsEnabledElement.IsEmpty == false && string.IsNullOrEmpty(webSocketsEnabledElement.Value) == false)
                        {
                            bool webSocketsEnabledInstance = bool.Parse(webSocketsEnabledElement.Value);
                            result.WebSocketsEnabled = webSocketsEnabledInstance;
                        }
                    }
                    
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        Tracing.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// You can retrieve historical usage metrics for a site by issuing an
        /// HTTP GET request.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/dn166964.aspx
        /// for more information)
        /// </summary>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Get Historical Usage Metrics
        /// Web Site operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The Get Historical Usage Metrics Web Site operation response.
        /// </returns>
        public async System.Threading.Tasks.Task<Microsoft.WindowsAzure.Management.WebSites.Models.WebSiteGetHistoricalUsageMetricsResponse> GetHistoricalUsageMetricsAsync(string webSpaceName, string webSiteName, WebSiteGetHistoricalUsageMetricsParameters parameters, CancellationToken cancellationToken)
        {
            // Validate
            if (webSpaceName == null)
            {
                throw new ArgumentNullException("webSpaceName");
            }
            if (webSiteName == null)
            {
                throw new ArgumentNullException("webSiteName");
            }
            if (parameters == null)
            {
                throw new ArgumentNullException("parameters");
            }
            
            // Tracing
            bool shouldTrace = CloudContext.Configuration.Tracing.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = Tracing.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("webSpaceName", webSpaceName);
                tracingParameters.Add("webSiteName", webSiteName);
                tracingParameters.Add("parameters", parameters);
                Tracing.Enter(invocationId, this, "GetHistoricalUsageMetricsAsync", tracingParameters);
            }
            
            // Construct URL
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            string url = "/" + this.Client.Credentials.SubscriptionId.Trim() + "/services/WebSpaces/" + webSpaceName.Trim() + "/sites/" + webSiteName.Trim() + "/metrics?";
            if (parameters.MetricNames != null && parameters.MetricNames.Count > 0)
            {
                url = url + "&names=" + Uri.EscapeUriString(string.Join(",", parameters.MetricNames));
            }
            if (parameters.StartTime != null)
            {
                url = url + "&StartTime=" + Uri.EscapeUriString(parameters.StartTime.Value.ToString());
            }
            if (parameters.EndTime != null)
            {
                url = url + "&EndTime=" + Uri.EscapeUriString(parameters.EndTime.Value.ToString());
            }
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-version", "2013-08-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        Tracing.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        Tracing.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false), CloudExceptionType.Xml);
                        if (shouldTrace)
                        {
                            Tracing.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    WebSiteGetHistoricalUsageMetricsResponse result = null;
                    // Deserialize Response
                    cancellationToken.ThrowIfCancellationRequested();
                    string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                    result = new WebSiteGetHistoricalUsageMetricsResponse();
                    XDocument responseDoc = XDocument.Parse(responseContent);
                    
                    XElement metricResponsesElement = responseDoc.Element(XName.Get("MetricResponses", "http://schemas.microsoft.com/windowsazure"));
                    if (metricResponsesElement != null && metricResponsesElement.IsEmpty == false)
                    {
                        if (metricResponsesElement != null && metricResponsesElement.IsEmpty == false)
                        {
                            foreach (XElement usageMetricsElement in metricResponsesElement.Elements(XName.Get("MetricResponse", "http://schemas.microsoft.com/windowsazure")))
                            {
                                WebSiteGetHistoricalUsageMetricsResponse.HistoricalUsageMetric metricResponseInstance = new WebSiteGetHistoricalUsageMetricsResponse.HistoricalUsageMetric();
                                result.UsageMetrics.Add(metricResponseInstance);
                                
                                XElement codeElement = usageMetricsElement.Element(XName.Get("Code", "http://schemas.microsoft.com/windowsazure"));
                                if (codeElement != null && codeElement.IsEmpty == false)
                                {
                                    string codeInstance = codeElement.Value;
                                    metricResponseInstance.Code = codeInstance;
                                }
                                
                                XElement dataElement = usageMetricsElement.Element(XName.Get("Data", "http://schemas.microsoft.com/windowsazure"));
                                if (dataElement != null && dataElement.IsEmpty == false)
                                {
                                    WebSiteGetHistoricalUsageMetricsResponse.HistoricalUsageMetricData dataInstance = new WebSiteGetHistoricalUsageMetricsResponse.HistoricalUsageMetricData();
                                    metricResponseInstance.Data = dataInstance;
                                    
                                    XElement displayNameElement = dataElement.Element(XName.Get("DisplayName", "http://schemas.microsoft.com/windowsazure"));
                                    if (displayNameElement != null && displayNameElement.IsEmpty == false)
                                    {
                                        string displayNameInstance = displayNameElement.Value;
                                        dataInstance.DisplayName = displayNameInstance;
                                    }
                                    
                                    XElement endTimeElement = dataElement.Element(XName.Get("EndTime", "http://schemas.microsoft.com/windowsazure"));
                                    if (endTimeElement != null && endTimeElement.IsEmpty == false)
                                    {
                                        DateTime endTimeInstance = DateTime.Parse(endTimeElement.Value, CultureInfo.InvariantCulture);
                                        dataInstance.EndTime = endTimeInstance;
                                    }
                                    
                                    XElement nameElement = dataElement.Element(XName.Get("Name", "http://schemas.microsoft.com/windowsazure"));
                                    if (nameElement != null && nameElement.IsEmpty == false)
                                    {
                                        string nameInstance = nameElement.Value;
                                        dataInstance.Name = nameInstance;
                                    }
                                    
                                    XElement primaryAggregationTypeElement = dataElement.Element(XName.Get("PrimaryAggregationType", "http://schemas.microsoft.com/windowsazure"));
                                    if (primaryAggregationTypeElement != null && primaryAggregationTypeElement.IsEmpty == false)
                                    {
                                        string primaryAggregationTypeInstance = primaryAggregationTypeElement.Value;
                                        dataInstance.PrimaryAggregationType = primaryAggregationTypeInstance;
                                    }
                                    
                                    XElement startTimeElement = dataElement.Element(XName.Get("StartTime", "http://schemas.microsoft.com/windowsazure"));
                                    if (startTimeElement != null && startTimeElement.IsEmpty == false)
                                    {
                                        DateTime startTimeInstance = DateTime.Parse(startTimeElement.Value, CultureInfo.InvariantCulture);
                                        dataInstance.StartTime = startTimeInstance;
                                    }
                                    
                                    XElement timeGrainElement = dataElement.Element(XName.Get("TimeGrain", "http://schemas.microsoft.com/windowsazure"));
                                    if (timeGrainElement != null && timeGrainElement.IsEmpty == false)
                                    {
                                        string timeGrainInstance = timeGrainElement.Value;
                                        dataInstance.TimeGrain = timeGrainInstance;
                                    }
                                    
                                    XElement unitElement = dataElement.Element(XName.Get("Unit", "http://schemas.microsoft.com/windowsazure"));
                                    if (unitElement != null && unitElement.IsEmpty == false)
                                    {
                                        string unitInstance = unitElement.Value;
                                        dataInstance.Unit = unitInstance;
                                    }
                                    
                                    XElement valuesSequenceElement = dataElement.Element(XName.Get("Values", "http://schemas.microsoft.com/windowsazure"));
                                    if (valuesSequenceElement != null && valuesSequenceElement.IsEmpty == false)
                                    {
                                        foreach (XElement valuesElement in valuesSequenceElement.Elements(XName.Get("MetricSample", "http://schemas.microsoft.com/windowsazure")))
                                        {
                                            WebSiteGetHistoricalUsageMetricsResponse.HistoricalUsageMetricSample metricSampleInstance = new WebSiteGetHistoricalUsageMetricsResponse.HistoricalUsageMetricSample();
                                            dataInstance.Values.Add(metricSampleInstance);
                                            
                                            XElement countElement = valuesElement.Element(XName.Get("Count", "http://schemas.microsoft.com/windowsazure"));
                                            if (countElement != null && countElement.IsEmpty == false)
                                            {
                                                int countInstance = int.Parse(countElement.Value, CultureInfo.InvariantCulture);
                                                metricSampleInstance.Count = countInstance;
                                            }
                                            
                                            XElement maximumElement = valuesElement.Element(XName.Get("Maximum", "http://schemas.microsoft.com/windowsazure"));
                                            if (maximumElement != null && maximumElement.IsEmpty == false)
                                            {
                                                bool isNil = false;
                                                XAttribute nilAttribute = maximumElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                                if (nilAttribute != null)
                                                {
                                                    isNil = nilAttribute.Value == "true";
                                                }
                                                if (isNil == false)
                                                {
                                                    string maximumInstance = maximumElement.Value;
                                                    metricSampleInstance.Maximum = maximumInstance;
                                                }
                                            }
                                            
                                            XElement minimumElement = valuesElement.Element(XName.Get("Minimum", "http://schemas.microsoft.com/windowsazure"));
                                            if (minimumElement != null && minimumElement.IsEmpty == false)
                                            {
                                                bool isNil2 = false;
                                                XAttribute nilAttribute2 = minimumElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                                if (nilAttribute2 != null)
                                                {
                                                    isNil2 = nilAttribute2.Value == "true";
                                                }
                                                if (isNil2 == false)
                                                {
                                                    string minimumInstance = minimumElement.Value;
                                                    metricSampleInstance.Minimum = minimumInstance;
                                                }
                                            }
                                            
                                            XElement timeCreatedElement = valuesElement.Element(XName.Get("TimeCreated", "http://schemas.microsoft.com/windowsazure"));
                                            if (timeCreatedElement != null && timeCreatedElement.IsEmpty == false)
                                            {
                                                DateTime timeCreatedInstance = DateTime.Parse(timeCreatedElement.Value, CultureInfo.InvariantCulture);
                                                metricSampleInstance.TimeCreated = timeCreatedInstance;
                                            }
                                            
                                            XElement totalElement = valuesElement.Element(XName.Get("Total", "http://schemas.microsoft.com/windowsazure"));
                                            if (totalElement != null && totalElement.IsEmpty == false)
                                            {
                                                string totalInstance = totalElement.Value;
                                                metricSampleInstance.Total = totalInstance;
                                            }
                                        }
                                    }
                                }
                                
                                XElement messageElement = usageMetricsElement.Element(XName.Get("Message", "http://schemas.microsoft.com/windowsazure"));
                                if (messageElement != null && messageElement.IsEmpty == false)
                                {
                                    string messageInstance = messageElement.Value;
                                    metricResponseInstance.Message = messageInstance;
                                }
                            }
                        }
                    }
                    
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        Tracing.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// You can retrieve the publish settings information for a web site by
        /// issuing an HTTP GET request.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/dn166996.aspx
        /// for more information)
        /// </summary>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The Get Publish Profile Web Site operation response.
        /// </returns>
        public async System.Threading.Tasks.Task<Microsoft.WindowsAzure.Management.WebSites.Models.WebSiteGetPublishProfileResponse> GetPublishProfileAsync(string webSpaceName, string webSiteName, CancellationToken cancellationToken)
        {
            // Validate
            if (webSpaceName == null)
            {
                throw new ArgumentNullException("webSpaceName");
            }
            if (webSiteName == null)
            {
                throw new ArgumentNullException("webSiteName");
            }
            
            // Tracing
            bool shouldTrace = CloudContext.Configuration.Tracing.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = Tracing.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("webSpaceName", webSpaceName);
                tracingParameters.Add("webSiteName", webSiteName);
                Tracing.Enter(invocationId, this, "GetPublishProfileAsync", tracingParameters);
            }
            
            // Construct URL
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            string url = "/" + this.Client.Credentials.SubscriptionId.Trim() + "/services/WebSpaces/" + webSpaceName.Trim() + "/sites/" + webSiteName.Trim() + "/publishxml";
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-version", "2013-08-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        Tracing.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        Tracing.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false), CloudExceptionType.Xml);
                        if (shouldTrace)
                        {
                            Tracing.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    WebSiteGetPublishProfileResponse result = null;
                    // Deserialize Response
                    cancellationToken.ThrowIfCancellationRequested();
                    string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                    result = new WebSiteGetPublishProfileResponse();
                    XDocument responseDoc = XDocument.Parse(responseContent);
                    
                    XElement publishDataElement = responseDoc.Element(XName.Get("publishData", ""));
                    if (publishDataElement != null && publishDataElement.IsEmpty == false)
                    {
                        if (publishDataElement != null && publishDataElement.IsEmpty == false)
                        {
                            foreach (XElement publishProfilesElement in publishDataElement.Elements(XName.Get("publishProfile", "")))
                            {
                                WebSiteGetPublishProfileResponse.PublishProfile publishProfileInstance = new WebSiteGetPublishProfileResponse.PublishProfile();
                                result.PublishProfiles.Add(publishProfileInstance);
                                
                                XAttribute profileNameAttribute = publishProfilesElement.Attribute(XName.Get("profileName", ""));
                                if (profileNameAttribute != null)
                                {
                                    publishProfileInstance.ProfileName = profileNameAttribute.Value;
                                }
                                
                                XAttribute publishMethodAttribute = publishProfilesElement.Attribute(XName.Get("publishMethod", ""));
                                if (publishMethodAttribute != null)
                                {
                                    publishProfileInstance.PublishMethod = publishMethodAttribute.Value;
                                }
                                
                                XAttribute publishUrlAttribute = publishProfilesElement.Attribute(XName.Get("publishUrl", ""));
                                if (publishUrlAttribute != null)
                                {
                                    publishProfileInstance.PublishUrl = publishUrlAttribute.Value;
                                }
                                
                                XAttribute msdeploySiteAttribute = publishProfilesElement.Attribute(XName.Get("msdeploySite", ""));
                                if (msdeploySiteAttribute != null)
                                {
                                    publishProfileInstance.MSDeploySite = msdeploySiteAttribute.Value;
                                }
                                
                                XAttribute ftpPassiveModeAttribute = publishProfilesElement.Attribute(XName.Get("ftpPassiveMode", ""));
                                if (ftpPassiveModeAttribute != null)
                                {
                                    publishProfileInstance.FtpPassiveMode = bool.Parse(ftpPassiveModeAttribute.Value);
                                }
                                
                                XAttribute userNameAttribute = publishProfilesElement.Attribute(XName.Get("userName", ""));
                                if (userNameAttribute != null)
                                {
                                    publishProfileInstance.UserName = userNameAttribute.Value;
                                }
                                
                                XAttribute userPWDAttribute = publishProfilesElement.Attribute(XName.Get("userPWD", ""));
                                if (userPWDAttribute != null)
                                {
                                    publishProfileInstance.UserPassword = userPWDAttribute.Value;
                                }
                                
                                XAttribute destinationAppUrlAttribute = publishProfilesElement.Attribute(XName.Get("destinationAppUrl", ""));
                                if (destinationAppUrlAttribute != null)
                                {
                                    publishProfileInstance.DestinationAppUri = TypeConversion.TryParseUri(destinationAppUrlAttribute.Value);
                                }
                                
                                XAttribute sQLServerDBConnectionStringAttribute = publishProfilesElement.Attribute(XName.Get("SQLServerDBConnectionString", ""));
                                if (sQLServerDBConnectionStringAttribute != null)
                                {
                                    publishProfileInstance.SqlServerConnectionString = sQLServerDBConnectionStringAttribute.Value;
                                }
                                
                                XAttribute mySQLDBConnectionStringAttribute = publishProfilesElement.Attribute(XName.Get("mySQLDBConnectionString", ""));
                                if (mySQLDBConnectionStringAttribute != null)
                                {
                                    publishProfileInstance.MySqlConnectionString = mySQLDBConnectionStringAttribute.Value;
                                }
                                
                                XAttribute hostingProviderForumLinkAttribute = publishProfilesElement.Attribute(XName.Get("hostingProviderForumLink", ""));
                                if (hostingProviderForumLinkAttribute != null)
                                {
                                    publishProfileInstance.HostingProviderForumUri = TypeConversion.TryParseUri(hostingProviderForumLinkAttribute.Value);
                                }
                                
                                XAttribute controlPanelLinkAttribute = publishProfilesElement.Attribute(XName.Get("controlPanelLink", ""));
                                if (controlPanelLinkAttribute != null)
                                {
                                    publishProfileInstance.ControlPanelUri = TypeConversion.TryParseUri(controlPanelLinkAttribute.Value);
                                }
                                
                                XElement databasesSequenceElement = publishProfilesElement.Element(XName.Get("databases", ""));
                                if (databasesSequenceElement != null && databasesSequenceElement.IsEmpty == false)
                                {
                                    foreach (XElement databasesElement in databasesSequenceElement.Elements(XName.Get("add", "")))
                                    {
                                        WebSiteGetPublishProfileResponse.Database addInstance = new WebSiteGetPublishProfileResponse.Database();
                                        publishProfileInstance.Databases.Add(addInstance);
                                        
                                        XAttribute nameAttribute = databasesElement.Attribute(XName.Get("name", ""));
                                        if (nameAttribute != null)
                                        {
                                            addInstance.Name = nameAttribute.Value;
                                        }
                                        
                                        XAttribute connectionStringAttribute = databasesElement.Attribute(XName.Get("connectionString", ""));
                                        if (connectionStringAttribute != null)
                                        {
                                            addInstance.ConnectionString = connectionStringAttribute.Value;
                                        }
                                        
                                        XAttribute providerNameAttribute = databasesElement.Attribute(XName.Get("providerName", ""));
                                        if (providerNameAttribute != null)
                                        {
                                            addInstance.ProviderName = providerNameAttribute.Value;
                                        }
                                        
                                        XAttribute typeAttribute = databasesElement.Attribute(XName.Get("type", ""));
                                        if (typeAttribute != null)
                                        {
                                            addInstance.Type = typeAttribute.Value;
                                        }
                                    }
                                }
                            }
                        }
                    }
                    
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        Tracing.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// A web site repository is essentially a Git repository that you can
        /// use to manage your web site content. By using Git source control
        /// tools, you can push or pull version-controlled changes to your
        /// site. This API executes a repository get operation.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/dn166967.aspx
        /// for more information)
        /// </summary>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The Get Repository Web Site operation response.
        /// </returns>
        public async System.Threading.Tasks.Task<Microsoft.WindowsAzure.Management.WebSites.Models.WebSiteGetRepositoryResponse> GetRepositoryAsync(string webSpaceName, string webSiteName, CancellationToken cancellationToken)
        {
            // Validate
            if (webSpaceName == null)
            {
                throw new ArgumentNullException("webSpaceName");
            }
            if (webSiteName == null)
            {
                throw new ArgumentNullException("webSiteName");
            }
            
            // Tracing
            bool shouldTrace = CloudContext.Configuration.Tracing.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = Tracing.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("webSpaceName", webSpaceName);
                tracingParameters.Add("webSiteName", webSiteName);
                Tracing.Enter(invocationId, this, "GetRepositoryAsync", tracingParameters);
            }
            
            // Construct URL
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            string url = "/" + this.Client.Credentials.SubscriptionId.Trim() + "/services/WebSpaces/" + webSpaceName.Trim() + "/sites/" + webSiteName.Trim() + "/repository";
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-version", "2013-08-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        Tracing.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        Tracing.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false), CloudExceptionType.Xml);
                        if (shouldTrace)
                        {
                            Tracing.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    WebSiteGetRepositoryResponse result = null;
                    // Deserialize Response
                    cancellationToken.ThrowIfCancellationRequested();
                    string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                    result = new WebSiteGetRepositoryResponse();
                    XDocument responseDoc = XDocument.Parse(responseContent);
                    
                    XElement anyURIElement = responseDoc.Element(XName.Get("anyURI", "http://schemas.microsoft.com/2003/10/Serialization/"));
                    if (anyURIElement != null && anyURIElement.IsEmpty == false)
                    {
                        result.Uri = TypeConversion.TryParseUri(anyURIElement.Value);
                    }
                    
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        Tracing.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// You can retrieve current usage metrics for a site by issuing an
        /// HTTP GET request. The metrics returned include CPU Time, Data In,
        /// Data Out, Local Bytes Read, Local Bytes Written, Network Bytes
        /// Read, Network Bytes Written, WP Stop Requests, Memory Usage, CPU
        /// Time - Minute Limit, and File System Storage.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/dn166991.aspx
        /// for more information)
        /// </summary>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The Get Usage Metrics Web Site operation response.
        /// </returns>
        public async System.Threading.Tasks.Task<Microsoft.WindowsAzure.Management.WebSites.Models.WebSiteGetUsageMetricsResponse> GetUsageMetricsAsync(string webSpaceName, string webSiteName, CancellationToken cancellationToken)
        {
            // Validate
            if (webSpaceName == null)
            {
                throw new ArgumentNullException("webSpaceName");
            }
            if (webSiteName == null)
            {
                throw new ArgumentNullException("webSiteName");
            }
            
            // Tracing
            bool shouldTrace = CloudContext.Configuration.Tracing.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = Tracing.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("webSpaceName", webSpaceName);
                tracingParameters.Add("webSiteName", webSiteName);
                Tracing.Enter(invocationId, this, "GetUsageMetricsAsync", tracingParameters);
            }
            
            // Construct URL
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            string url = "/" + this.Client.Credentials.SubscriptionId.Trim() + "/services/WebSpaces/" + webSpaceName.Trim() + "/sites/" + webSiteName.Trim() + "/usages";
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-version", "2013-08-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        Tracing.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        Tracing.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false), CloudExceptionType.Xml);
                        if (shouldTrace)
                        {
                            Tracing.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    WebSiteGetUsageMetricsResponse result = null;
                    // Deserialize Response
                    cancellationToken.ThrowIfCancellationRequested();
                    string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                    result = new WebSiteGetUsageMetricsResponse();
                    XDocument responseDoc = XDocument.Parse(responseContent);
                    
                    XElement usagesElement = responseDoc.Element(XName.Get("Usages", "http://schemas.microsoft.com/windowsazure"));
                    if (usagesElement != null && usagesElement.IsEmpty == false)
                    {
                        if (usagesElement != null && usagesElement.IsEmpty == false)
                        {
                            foreach (XElement usageMetricsElement in usagesElement.Elements(XName.Get("Usage", "http://schemas.microsoft.com/windowsazure")))
                            {
                                WebSiteGetUsageMetricsResponse.UsageMetric usageInstance = new WebSiteGetUsageMetricsResponse.UsageMetric();
                                result.UsageMetrics.Add(usageInstance);
                                
                                XElement computeModeElement = usageMetricsElement.Element(XName.Get("ComputeMode", "http://schemas.microsoft.com/windowsazure"));
                                if (computeModeElement != null && computeModeElement.IsEmpty == false)
                                {
                                    WebSiteComputeMode computeModeInstance = ((WebSiteComputeMode)Enum.Parse(typeof(WebSiteComputeMode), computeModeElement.Value, true));
                                    usageInstance.ComputeMode = computeModeInstance;
                                }
                                
                                XElement currentValueElement = usageMetricsElement.Element(XName.Get("CurrentValue", "http://schemas.microsoft.com/windowsazure"));
                                if (currentValueElement != null && currentValueElement.IsEmpty == false)
                                {
                                    string currentValueInstance = currentValueElement.Value;
                                    usageInstance.CurrentValue = currentValueInstance;
                                }
                                
                                XElement displayNameElement = usageMetricsElement.Element(XName.Get("DisplayName", "http://schemas.microsoft.com/windowsazure"));
                                if (displayNameElement != null && displayNameElement.IsEmpty == false)
                                {
                                    string displayNameInstance = displayNameElement.Value;
                                    usageInstance.DisplayName = displayNameInstance;
                                }
                                
                                XElement limitElement = usageMetricsElement.Element(XName.Get("Limit", "http://schemas.microsoft.com/windowsazure"));
                                if (limitElement != null && limitElement.IsEmpty == false)
                                {
                                    string limitInstance = limitElement.Value;
                                    usageInstance.Limit = limitInstance;
                                }
                                
                                XElement nameElement = usageMetricsElement.Element(XName.Get("Name", "http://schemas.microsoft.com/windowsazure"));
                                if (nameElement != null && nameElement.IsEmpty == false)
                                {
                                    string nameInstance = nameElement.Value;
                                    usageInstance.Name = nameInstance;
                                }
                                
                                XElement nextResetTimeElement = usageMetricsElement.Element(XName.Get("NextResetTime", "http://schemas.microsoft.com/windowsazure"));
                                if (nextResetTimeElement != null && nextResetTimeElement.IsEmpty == false)
                                {
                                    DateTime nextResetTimeInstance = DateTime.Parse(nextResetTimeElement.Value, CultureInfo.InvariantCulture);
                                    usageInstance.NextResetTime = nextResetTimeInstance;
                                }
                                
                                XElement resourceNameElement = usageMetricsElement.Element(XName.Get("ResourceName", "http://schemas.microsoft.com/windowsazure"));
                                if (resourceNameElement != null && resourceNameElement.IsEmpty == false)
                                {
                                    string resourceNameInstance = resourceNameElement.Value;
                                    usageInstance.ResourceName = resourceNameInstance;
                                }
                                
                                XElement siteModeElement = usageMetricsElement.Element(XName.Get("SiteMode", "http://schemas.microsoft.com/windowsazure"));
                                if (siteModeElement != null && siteModeElement.IsEmpty == false)
                                {
                                    WebSiteMode siteModeInstance = ((WebSiteMode)Enum.Parse(typeof(WebSiteMode), siteModeElement.Value, true));
                                    usageInstance.SiteMode = siteModeInstance;
                                }
                                
                                XElement unitElement = usageMetricsElement.Element(XName.Get("Unit", "http://schemas.microsoft.com/windowsazure"));
                                if (unitElement != null && unitElement.IsEmpty == false)
                                {
                                    string unitInstance = unitElement.Value;
                                    usageInstance.Unit = unitInstance;
                                }
                            }
                        }
                    }
                    
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        Tracing.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Determines if a host name is available.
        /// </summary>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The Is Hostname Available Web Site operation response.
        /// </returns>
        public async System.Threading.Tasks.Task<Microsoft.WindowsAzure.Management.WebSites.Models.WebSiteIsHostnameAvailableResponse> IsHostnameAvailableAsync(string webSiteName, CancellationToken cancellationToken)
        {
            // Validate
            if (webSiteName == null)
            {
                throw new ArgumentNullException("webSiteName");
            }
            
            // Tracing
            bool shouldTrace = CloudContext.Configuration.Tracing.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = Tracing.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("webSiteName", webSiteName);
                Tracing.Enter(invocationId, this, "IsHostnameAvailableAsync", tracingParameters);
            }
            
            // Construct URL
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            string url = "/" + this.Client.Credentials.SubscriptionId.Trim() + "/services/WebSpaces?";
            url = url + "ishostnameavailable=" + Uri.EscapeUriString(webSiteName.Trim());
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-version", "2013-08-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        Tracing.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        Tracing.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false), CloudExceptionType.Xml);
                        if (shouldTrace)
                        {
                            Tracing.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    WebSiteIsHostnameAvailableResponse result = null;
                    // Deserialize Response
                    cancellationToken.ThrowIfCancellationRequested();
                    string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                    result = new WebSiteIsHostnameAvailableResponse();
                    XDocument responseDoc = XDocument.Parse(responseContent);
                    
                    XElement booleanElement = responseDoc.Element(XName.Get("boolean", "http://schemas.microsoft.com/2003/10/Serialization/"));
                    if (booleanElement != null && booleanElement.IsEmpty == false)
                    {
                        bool booleanInstance = bool.Parse(booleanElement.Value);
                        result.IsAvailable = booleanInstance;
                    }
                    
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        Tracing.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// You can restart a web site by issuing an HTTP POST request.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/dn236425.aspx
        /// for more information)
        /// </summary>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public async System.Threading.Tasks.Task<OperationResponse> RestartAsync(string webSpaceName, string webSiteName, CancellationToken cancellationToken)
        {
            // Validate
            if (webSpaceName == null)
            {
                throw new ArgumentNullException("webSpaceName");
            }
            if (webSiteName == null)
            {
                throw new ArgumentNullException("webSiteName");
            }
            
            // Tracing
            bool shouldTrace = CloudContext.Configuration.Tracing.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = Tracing.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("webSpaceName", webSpaceName);
                tracingParameters.Add("webSiteName", webSiteName);
                Tracing.Enter(invocationId, this, "RestartAsync", tracingParameters);
            }
            
            // Construct URL
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            string url = "/" + this.Client.Credentials.SubscriptionId.Trim() + "/services/WebSpaces/" + webSpaceName.Trim() + "/sites/" + webSiteName.Trim() + "/restart";
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Post;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-version", "2013-08-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        Tracing.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        Tracing.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false), CloudExceptionType.Xml);
                        if (shouldTrace)
                        {
                            Tracing.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    OperationResponse result = null;
                    result = new OperationResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        Tracing.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// You can swap a web site from one slot to the production slot.
        /// </summary>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <param name='slotName'>
        /// Required. The name of the web site slot to swap with the production
        /// slot.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified long-running
        /// operation, indicating whether it has succeeded, is inprogress, has
        /// timed out, or has failed. Note that this status is distinct from
        /// the HTTP status code returned for the Get Operation Status
        /// operation itself. If the long-running operation failed, the
        /// response body includes error information regarding the failure.
        /// </returns>
        public async System.Threading.Tasks.Task<Microsoft.WindowsAzure.Management.WebSites.Models.WebSiteOperationStatusResponse> SwapSlotsAsync(string webSpaceName, string webSiteName, string slotName, CancellationToken cancellationToken)
        {
            WebSiteManagementClient client = this.Client;
            bool shouldTrace = CloudContext.Configuration.Tracing.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = Tracing.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("webSpaceName", webSpaceName);
                tracingParameters.Add("webSiteName", webSiteName);
                tracingParameters.Add("slotName", slotName);
                Tracing.Enter(invocationId, this, "SwapSlotsAsync", tracingParameters);
            }
            try
            {
                if (shouldTrace)
                {
                    client = this.Client.WithHandler(new ClientRequestTrackingHandler(invocationId));
                }
                
                cancellationToken.ThrowIfCancellationRequested();
                WebSiteOperationStatusResponse response = await client.WebSites.BeginSwapingSlotsAsync(webSpaceName, webSiteName, slotName, cancellationToken).ConfigureAwait(false);
                if (response.Status == WebSiteOperationStatus.Succeeded)
                {
                    return response;
                }
                cancellationToken.ThrowIfCancellationRequested();
                WebSiteOperationStatusResponse result = await client.GetOperationStatusAsync(webSpaceName, webSiteName, response.OperationId, cancellationToken).ConfigureAwait(false);
                int delayInSeconds = 30;
                while ((result.Status != WebSiteOperationStatus.InProgress) == false)
                {
                    cancellationToken.ThrowIfCancellationRequested();
                    await TaskEx.Delay(delayInSeconds * 1000, cancellationToken).ConfigureAwait(false);
                    cancellationToken.ThrowIfCancellationRequested();
                    result = await client.GetOperationStatusAsync(webSpaceName, webSiteName, response.OperationId, cancellationToken).ConfigureAwait(false);
                    delayInSeconds = 30;
                }
                
                if (shouldTrace)
                {
                    Tracing.Exit(invocationId, result);
                }
                
                if (result.Status != WebSiteOperationStatus.Succeeded)
                {
                    if (result.Errors != null && result.Errors.Count > 0)
                    {
                        CloudException ex = new CloudException(result.Errors.ElementAtOrDefault(0).Code + " : " + result.Errors.ElementAtOrDefault(0).Message);
                        ex.ErrorCode = result.Errors.ElementAtOrDefault(0).Code;
                        ex.ErrorMessage = result.Errors.ElementAtOrDefault(0).Message;
                        if (shouldTrace)
                        {
                            Tracing.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    else
                    {
                        CloudException ex = new CloudException("");
                        if (shouldTrace)
                        {
                            Tracing.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                }
                
                return result;
            }
            finally
            {
                if (client != null && shouldTrace)
                {
                    client.Dispose();
                }
            }
        }
        
        /// <summary>
        /// A web site repository is essentially a Git repository that you can
        /// use to manage your web site content. By using Git source control
        /// tools, you can push or pull version-controlled changes to your
        /// site. This API executes a repository sync operation.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/dn166967.aspx
        /// for more information)
        /// </summary>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public async System.Threading.Tasks.Task<OperationResponse> SyncRepositoryAsync(string webSpaceName, string webSiteName, CancellationToken cancellationToken)
        {
            // Validate
            if (webSpaceName == null)
            {
                throw new ArgumentNullException("webSpaceName");
            }
            if (webSiteName == null)
            {
                throw new ArgumentNullException("webSiteName");
            }
            
            // Tracing
            bool shouldTrace = CloudContext.Configuration.Tracing.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = Tracing.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("webSpaceName", webSpaceName);
                tracingParameters.Add("webSiteName", webSiteName);
                Tracing.Enter(invocationId, this, "SyncRepositoryAsync", tracingParameters);
            }
            
            // Construct URL
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            string url = "/" + this.Client.Credentials.SubscriptionId.Trim() + "/services/WebSpaces/" + webSpaceName.Trim() + "/sites/" + webSiteName.Trim() + "/repository?action=sync";
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Post;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-version", "2013-08-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        Tracing.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        Tracing.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false), CloudExceptionType.Xml);
                        if (shouldTrace)
                        {
                            Tracing.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    OperationResponse result = null;
                    result = new OperationResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        Tracing.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// You can update the settings for a web site by using the HTTP PUT
        /// method and by specifying the settings in the request body.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/dn167005.aspx
        /// for more information)
        /// </summary>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Update Web Site operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The Update Web Site operation response.
        /// </returns>
        public async System.Threading.Tasks.Task<Microsoft.WindowsAzure.Management.WebSites.Models.WebSiteUpdateResponse> UpdateAsync(string webSpaceName, string webSiteName, WebSiteUpdateParameters parameters, CancellationToken cancellationToken)
        {
            // Validate
            if (webSpaceName == null)
            {
                throw new ArgumentNullException("webSpaceName");
            }
            if (webSiteName == null)
            {
                throw new ArgumentNullException("webSiteName");
            }
            if (parameters == null)
            {
                throw new ArgumentNullException("parameters");
            }
            
            // Tracing
            bool shouldTrace = CloudContext.Configuration.Tracing.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = Tracing.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("webSpaceName", webSpaceName);
                tracingParameters.Add("webSiteName", webSiteName);
                tracingParameters.Add("parameters", parameters);
                Tracing.Enter(invocationId, this, "UpdateAsync", tracingParameters);
            }
            
            // Construct URL
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            string url = "/" + this.Client.Credentials.SubscriptionId.Trim() + "/services/WebSpaces/" + webSpaceName.Trim() + "/sites/" + webSiteName.Trim();
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Put;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-version", "2013-08-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Serialize Request
                string requestContent = null;
                XDocument requestDoc = new XDocument();
                
                XElement siteElement = new XElement(XName.Get("Site", "http://schemas.microsoft.com/windowsazure"));
                requestDoc.Add(siteElement);
                
                if (parameters.AvailabilityState != null)
                {
                    XElement availabilityStateElement = new XElement(XName.Get("AvailabilityState", "http://schemas.microsoft.com/windowsazure"));
                    availabilityStateElement.Value = parameters.AvailabilityState.ToString();
                    siteElement.Add(availabilityStateElement);
                }
                
                if (parameters.ComputeMode != null)
                {
                    XElement computeModeElement = new XElement(XName.Get("ComputeMode", "http://schemas.microsoft.com/windowsazure"));
                    computeModeElement.Value = parameters.ComputeMode.ToString();
                    siteElement.Add(computeModeElement);
                }
                
                if (parameters.Enabled != null)
                {
                    XElement enabledElement = new XElement(XName.Get("Enabled", "http://schemas.microsoft.com/windowsazure"));
                    enabledElement.Value = parameters.Enabled.ToString().ToLower();
                    siteElement.Add(enabledElement);
                }
                
                if (parameters.HostNameSslStates != null)
                {
                    XElement hostNameSslStatesSequenceElement = new XElement(XName.Get("HostNameSslStates", "http://schemas.microsoft.com/windowsazure"));
                    foreach (WebSiteUpdateParameters.WebSiteHostNameSslState hostNameSslStatesItem in parameters.HostNameSslStates)
                    {
                        XElement webSiteHostNameSslStateElement = new XElement(XName.Get("WebSiteHostNameSslState", "http://schemas.microsoft.com/windowsazure"));
                        hostNameSslStatesSequenceElement.Add(webSiteHostNameSslStateElement);
                        
                        if (hostNameSslStatesItem.SslState != null)
                        {
                            XElement sslStateElement = new XElement(XName.Get("SslState", "http://schemas.microsoft.com/windowsazure"));
                            sslStateElement.Value = hostNameSslStatesItem.SslState.ToString();
                            webSiteHostNameSslStateElement.Add(sslStateElement);
                        }
                        
                        if (hostNameSslStatesItem.Thumbprint != null)
                        {
                            XElement thumbprintElement = new XElement(XName.Get("Thumbprint", "http://schemas.microsoft.com/windowsazure"));
                            thumbprintElement.Value = hostNameSslStatesItem.Thumbprint;
                            webSiteHostNameSslStateElement.Add(thumbprintElement);
                        }
                        else
                        {
                            XElement emptyElement = new XElement(XName.Get("Thumbprint", "http://schemas.microsoft.com/windowsazure"));
                            XAttribute nilAttribute = new XAttribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"), "");
                            nilAttribute.Value = "true";
                            emptyElement.Add(nilAttribute);
                            webSiteHostNameSslStateElement.Add(emptyElement);
                        }
                        
                        if (hostNameSslStatesItem.ToUpdate != null)
                        {
                            XElement toUpdateElement = new XElement(XName.Get("ToUpdate", "http://schemas.microsoft.com/windowsazure"));
                            toUpdateElement.Value = hostNameSslStatesItem.ToUpdate.ToString().ToLower();
                            webSiteHostNameSslStateElement.Add(toUpdateElement);
                        }
                    }
                    siteElement.Add(hostNameSslStatesSequenceElement);
                }
                
                if (parameters.HostNames != null)
                {
                    XElement hostNamesSequenceElement = new XElement(XName.Get("HostNames", "http://schemas.microsoft.com/windowsazure"));
                    foreach (string hostNamesItem in parameters.HostNames)
                    {
                        XElement hostNamesItemElement = new XElement(XName.Get("string", "http://schemas.microsoft.com/2003/10/Serialization/Arrays"));
                        hostNamesItemElement.Value = hostNamesItem;
                        hostNamesSequenceElement.Add(hostNamesItemElement);
                    }
                    siteElement.Add(hostNamesSequenceElement);
                }
                
                if (parameters.RuntimeAvailabilityState != null)
                {
                    XElement runtimeAvailabilityStateElement = new XElement(XName.Get("RuntimeAvailabilityState", "http://schemas.microsoft.com/windowsazure"));
                    runtimeAvailabilityStateElement.Value = parameters.RuntimeAvailabilityState.ToString();
                    siteElement.Add(runtimeAvailabilityStateElement);
                }
                
                if (parameters.SslCertificates != null)
                {
                    XElement sSLCertificatesSequenceElement = new XElement(XName.Get("SSLCertificates", "http://schemas.microsoft.com/windowsazure"));
                    foreach (WebSiteUpdateParameters.WebSiteSslCertificate sSLCertificatesItem in parameters.SslCertificates)
                    {
                        XElement certificateElement = new XElement(XName.Get("Certificate", "http://schemas.microsoft.com/windowsazure"));
                        sSLCertificatesSequenceElement.Add(certificateElement);
                        
                        if (sSLCertificatesItem.Password != null)
                        {
                            XElement passwordElement = new XElement(XName.Get("Password", "http://schemas.microsoft.com/windowsazure"));
                            passwordElement.Value = sSLCertificatesItem.Password;
                            certificateElement.Add(passwordElement);
                        }
                        
                        if (sSLCertificatesItem.PfxBlob != null)
                        {
                            XElement pfxBlobElement = new XElement(XName.Get("PfxBlob", "http://schemas.microsoft.com/windowsazure"));
                            pfxBlobElement.Value = Convert.ToBase64String(sSLCertificatesItem.PfxBlob);
                            certificateElement.Add(pfxBlobElement);
                        }
                        
                        if (sSLCertificatesItem.Thumbprint != null)
                        {
                            XElement thumbprintElement2 = new XElement(XName.Get("Thumbprint", "http://schemas.microsoft.com/windowsazure"));
                            thumbprintElement2.Value = sSLCertificatesItem.Thumbprint;
                            certificateElement.Add(thumbprintElement2);
                        }
                        
                        if (sSLCertificatesItem.IsToBeDeleted != null)
                        {
                            XElement toDeleteElement = new XElement(XName.Get("ToDelete", "http://schemas.microsoft.com/windowsazure"));
                            toDeleteElement.Value = sSLCertificatesItem.IsToBeDeleted.ToString().ToLower();
                            certificateElement.Add(toDeleteElement);
                        }
                    }
                    siteElement.Add(sSLCertificatesSequenceElement);
                }
                
                if (parameters.ServerFarm != null)
                {
                    XElement serverFarmElement = new XElement(XName.Get("ServerFarm", "http://schemas.microsoft.com/windowsazure"));
                    serverFarmElement.Value = parameters.ServerFarm;
                    siteElement.Add(serverFarmElement);
                }
                
                if (parameters.SiteMode != null)
                {
                    XElement siteModeElement = new XElement(XName.Get("SiteMode", "http://schemas.microsoft.com/windowsazure"));
                    siteModeElement.Value = parameters.SiteMode.ToString();
                    siteElement.Add(siteModeElement);
                }
                
                if (parameters.State != null)
                {
                    XElement stateElement = new XElement(XName.Get("State", "http://schemas.microsoft.com/windowsazure"));
                    stateElement.Value = parameters.State;
                    siteElement.Add(stateElement);
                }
                
                requestContent = requestDoc.ToString();
                httpRequest.Content = new StringContent(requestContent, Encoding.UTF8);
                httpRequest.Content.Headers.ContentType = new MediaTypeHeaderValue("application/xml");
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        Tracing.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        Tracing.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, requestContent, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false), CloudExceptionType.Xml);
                        if (shouldTrace)
                        {
                            Tracing.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    WebSiteUpdateResponse result = null;
                    // Deserialize Response
                    cancellationToken.ThrowIfCancellationRequested();
                    string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                    result = new WebSiteUpdateResponse();
                    XDocument responseDoc = XDocument.Parse(responseContent);
                    
                    XElement siteElement2 = responseDoc.Element(XName.Get("Site", "http://schemas.microsoft.com/windowsazure"));
                    if (siteElement2 != null && siteElement2.IsEmpty == false)
                    {
                        WebSite webSiteInstance = new WebSite();
                        result.WebSite = webSiteInstance;
                        
                        XElement adminEnabledElement = siteElement2.Element(XName.Get("AdminEnabled", "http://schemas.microsoft.com/windowsazure"));
                        if (adminEnabledElement != null && adminEnabledElement.IsEmpty == false)
                        {
                            bool adminEnabledInstance = bool.Parse(adminEnabledElement.Value);
                            webSiteInstance.AdminEnabled = adminEnabledInstance;
                        }
                        
                        XElement availabilityStateElement2 = siteElement2.Element(XName.Get("AvailabilityState", "http://schemas.microsoft.com/windowsazure"));
                        if (availabilityStateElement2 != null && availabilityStateElement2.IsEmpty == false)
                        {
                            WebSpaceAvailabilityState availabilityStateInstance = ((WebSpaceAvailabilityState)Enum.Parse(typeof(WebSpaceAvailabilityState), availabilityStateElement2.Value, true));
                            webSiteInstance.AvailabilityState = availabilityStateInstance;
                        }
                        
                        XElement computeModeElement2 = siteElement2.Element(XName.Get("ComputeMode", "http://schemas.microsoft.com/windowsazure"));
                        if (computeModeElement2 != null && computeModeElement2.IsEmpty == false)
                        {
                            WebSiteComputeMode computeModeInstance = ((WebSiteComputeMode)Enum.Parse(typeof(WebSiteComputeMode), computeModeElement2.Value, true));
                            webSiteInstance.ComputeMode = computeModeInstance;
                        }
                        
                        XElement enabledElement2 = siteElement2.Element(XName.Get("Enabled", "http://schemas.microsoft.com/windowsazure"));
                        if (enabledElement2 != null && enabledElement2.IsEmpty == false)
                        {
                            bool enabledInstance = bool.Parse(enabledElement2.Value);
                            webSiteInstance.Enabled = enabledInstance;
                        }
                        
                        XElement enabledHostNamesSequenceElement = siteElement2.Element(XName.Get("EnabledHostNames", "http://schemas.microsoft.com/windowsazure"));
                        if (enabledHostNamesSequenceElement != null && enabledHostNamesSequenceElement.IsEmpty == false)
                        {
                            foreach (XElement enabledHostNamesElement in enabledHostNamesSequenceElement.Elements(XName.Get("string", "http://schemas.microsoft.com/2003/10/Serialization/Arrays")))
                            {
                                webSiteInstance.EnabledHostNames.Add(enabledHostNamesElement.Value);
                            }
                        }
                        
                        XElement hostNameSslStatesSequenceElement2 = siteElement2.Element(XName.Get("HostNameSslStates", "http://schemas.microsoft.com/windowsazure"));
                        if (hostNameSslStatesSequenceElement2 != null && hostNameSslStatesSequenceElement2.IsEmpty == false)
                        {
                            foreach (XElement hostNameSslStatesElement in hostNameSslStatesSequenceElement2.Elements(XName.Get("WebSiteHostNameSslState", "http://schemas.microsoft.com/windowsazure")))
                            {
                                WebSite.WebSiteHostNameSslState webSiteHostNameSslStateInstance = new WebSite.WebSiteHostNameSslState();
                                webSiteInstance.HostNameSslStates.Add(webSiteHostNameSslStateInstance);
                                
                                XElement nameElement = hostNameSslStatesElement.Element(XName.Get("Name", "http://schemas.microsoft.com/windowsazure"));
                                if (nameElement != null && nameElement.IsEmpty == false)
                                {
                                    string nameInstance = nameElement.Value;
                                    webSiteHostNameSslStateInstance.Name = nameInstance;
                                }
                                
                                XElement sslStateElement2 = hostNameSslStatesElement.Element(XName.Get("SslState", "http://schemas.microsoft.com/windowsazure"));
                                if (sslStateElement2 != null && sslStateElement2.IsEmpty == false)
                                {
                                    WebSiteSslState sslStateInstance = ((WebSiteSslState)Enum.Parse(typeof(WebSiteSslState), sslStateElement2.Value, true));
                                    webSiteHostNameSslStateInstance.SslState = sslStateInstance;
                                }
                                
                                XElement thumbprintElement3 = hostNameSslStatesElement.Element(XName.Get("Thumbprint", "http://schemas.microsoft.com/windowsazure"));
                                if (thumbprintElement3 != null && thumbprintElement3.IsEmpty == false)
                                {
                                    bool isNil = false;
                                    XAttribute nilAttribute2 = thumbprintElement3.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                    if (nilAttribute2 != null)
                                    {
                                        isNil = nilAttribute2.Value == "true";
                                    }
                                    if (isNil == false)
                                    {
                                        string thumbprintInstance = thumbprintElement3.Value;
                                        webSiteHostNameSslStateInstance.Thumbprint = thumbprintInstance;
                                    }
                                }
                                
                                XElement virtualIPElement = hostNameSslStatesElement.Element(XName.Get("VirtualIP", "http://schemas.microsoft.com/windowsazure"));
                                if (virtualIPElement != null && virtualIPElement.IsEmpty == false)
                                {
                                    bool isNil2 = false;
                                    XAttribute nilAttribute3 = virtualIPElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                    if (nilAttribute3 != null)
                                    {
                                        isNil2 = nilAttribute3.Value == "true";
                                    }
                                    if (isNil2 == false)
                                    {
                                        string virtualIPInstance = virtualIPElement.Value;
                                        webSiteHostNameSslStateInstance.VirtualIP = virtualIPInstance;
                                    }
                                }
                            }
                        }
                        
                        XElement hostNamesSequenceElement2 = siteElement2.Element(XName.Get("HostNames", "http://schemas.microsoft.com/windowsazure"));
                        if (hostNamesSequenceElement2 != null && hostNamesSequenceElement2.IsEmpty == false)
                        {
                            foreach (XElement hostNamesElement in hostNamesSequenceElement2.Elements(XName.Get("string", "http://schemas.microsoft.com/2003/10/Serialization/Arrays")))
                            {
                                webSiteInstance.HostNames.Add(hostNamesElement.Value);
                            }
                        }
                        
                        XElement lastModifiedTimeUtcElement = siteElement2.Element(XName.Get("LastModifiedTimeUtc", "http://schemas.microsoft.com/windowsazure"));
                        if (lastModifiedTimeUtcElement != null && lastModifiedTimeUtcElement.IsEmpty == false)
                        {
                            DateTime lastModifiedTimeUtcInstance = DateTime.Parse(lastModifiedTimeUtcElement.Value, CultureInfo.InvariantCulture);
                            webSiteInstance.LastModifiedTimeUtc = lastModifiedTimeUtcInstance;
                        }
                        
                        XElement nameElement2 = siteElement2.Element(XName.Get("Name", "http://schemas.microsoft.com/windowsazure"));
                        if (nameElement2 != null && nameElement2.IsEmpty == false)
                        {
                            string nameInstance2 = nameElement2.Value;
                            webSiteInstance.Name = nameInstance2;
                        }
                        
                        XElement ownerElement = siteElement2.Element(XName.Get("Owner", "http://schemas.microsoft.com/windowsazure"));
                        if (ownerElement != null && ownerElement.IsEmpty == false)
                        {
                            bool isNil3 = false;
                            XAttribute nilAttribute4 = ownerElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                            if (nilAttribute4 != null)
                            {
                                isNil3 = nilAttribute4.Value == "true";
                            }
                            if (isNil3 == false)
                            {
                                string ownerInstance = ownerElement.Value;
                                webSiteInstance.Owner = ownerInstance;
                            }
                        }
                        
                        XElement repositorySiteNameElement = siteElement2.Element(XName.Get("RepositorySiteName", "http://schemas.microsoft.com/windowsazure"));
                        if (repositorySiteNameElement != null && repositorySiteNameElement.IsEmpty == false)
                        {
                            string repositorySiteNameInstance = repositorySiteNameElement.Value;
                            webSiteInstance.RepositorySiteName = repositorySiteNameInstance;
                        }
                        
                        XElement runtimeAvailabilityStateElement2 = siteElement2.Element(XName.Get("RuntimeAvailabilityState", "http://schemas.microsoft.com/windowsazure"));
                        if (runtimeAvailabilityStateElement2 != null && runtimeAvailabilityStateElement2.IsEmpty == false)
                        {
                            WebSiteRuntimeAvailabilityState runtimeAvailabilityStateInstance = ((WebSiteRuntimeAvailabilityState)Enum.Parse(typeof(WebSiteRuntimeAvailabilityState), runtimeAvailabilityStateElement2.Value, true));
                            webSiteInstance.RuntimeAvailabilityState = runtimeAvailabilityStateInstance;
                        }
                        
                        XElement sSLCertificatesSequenceElement2 = siteElement2.Element(XName.Get("SSLCertificates", "http://schemas.microsoft.com/windowsazure"));
                        if (sSLCertificatesSequenceElement2 != null && sSLCertificatesSequenceElement2.IsEmpty == false)
                        {
                            bool isNil4 = false;
                            XAttribute nilAttribute5 = sSLCertificatesSequenceElement2.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                            if (nilAttribute5 != null)
                            {
                                isNil4 = nilAttribute5.Value == "true";
                            }
                            if (isNil4 == false)
                            {
                                foreach (XElement sSLCertificatesElement in sSLCertificatesSequenceElement2.Elements(XName.Get("Certificate", "http://schemas.microsoft.com/windowsazure")))
                                {
                                    WebSite.WebSiteSslCertificate certificateInstance = new WebSite.WebSiteSslCertificate();
                                    webSiteInstance.SslCertificates.Add(certificateInstance);
                                    
                                    XElement expirationDateElement = sSLCertificatesElement.Element(XName.Get("ExpirationDate", "http://schemas.microsoft.com/windowsazure"));
                                    if (expirationDateElement != null && expirationDateElement.IsEmpty == false && string.IsNullOrEmpty(expirationDateElement.Value) == false)
                                    {
                                        bool isNil5 = false;
                                        XAttribute nilAttribute6 = expirationDateElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                        if (nilAttribute6 != null)
                                        {
                                            isNil5 = nilAttribute6.Value == "true";
                                        }
                                        if (isNil5 == false)
                                        {
                                            DateTime expirationDateInstance = DateTime.Parse(expirationDateElement.Value, CultureInfo.InvariantCulture);
                                            certificateInstance.ExpirationDate = expirationDateInstance;
                                        }
                                    }
                                    
                                    XElement friendlyNameElement = sSLCertificatesElement.Element(XName.Get("FriendlyName", "http://schemas.microsoft.com/windowsazure"));
                                    if (friendlyNameElement != null && friendlyNameElement.IsEmpty == false)
                                    {
                                        bool isNil6 = false;
                                        XAttribute nilAttribute7 = friendlyNameElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                        if (nilAttribute7 != null)
                                        {
                                            isNil6 = nilAttribute7.Value == "true";
                                        }
                                        if (isNil6 == false)
                                        {
                                            string friendlyNameInstance = friendlyNameElement.Value;
                                            certificateInstance.FriendlyName = friendlyNameInstance;
                                        }
                                    }
                                    
                                    XElement hostNamesSequenceElement3 = sSLCertificatesElement.Element(XName.Get("HostNames", "http://schemas.microsoft.com/windowsazure"));
                                    if (hostNamesSequenceElement3 != null && hostNamesSequenceElement3.IsEmpty == false)
                                    {
                                        bool isNil7 = false;
                                        XAttribute nilAttribute8 = hostNamesSequenceElement3.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                        if (nilAttribute8 != null)
                                        {
                                            isNil7 = nilAttribute8.Value == "true";
                                        }
                                        if (isNil7 == false)
                                        {
                                            foreach (XElement hostNamesElement2 in hostNamesSequenceElement3.Elements(XName.Get("string", "http://schemas.microsoft.com/2003/10/Serialization/Arrays")))
                                            {
                                                certificateInstance.HostNames.Add(hostNamesElement2.Value);
                                            }
                                        }
                                    }
                                    
                                    XElement issueDateElement = sSLCertificatesElement.Element(XName.Get("IssueDate", "http://schemas.microsoft.com/windowsazure"));
                                    if (issueDateElement != null && issueDateElement.IsEmpty == false && string.IsNullOrEmpty(issueDateElement.Value) == false)
                                    {
                                        bool isNil8 = false;
                                        XAttribute nilAttribute9 = issueDateElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                        if (nilAttribute9 != null)
                                        {
                                            isNil8 = nilAttribute9.Value == "true";
                                        }
                                        if (isNil8 == false)
                                        {
                                            DateTime issueDateInstance = DateTime.Parse(issueDateElement.Value, CultureInfo.InvariantCulture);
                                            certificateInstance.IssueDate = issueDateInstance;
                                        }
                                    }
                                    
                                    XElement issuerElement = sSLCertificatesElement.Element(XName.Get("Issuer", "http://schemas.microsoft.com/windowsazure"));
                                    if (issuerElement != null && issuerElement.IsEmpty == false)
                                    {
                                        bool isNil9 = false;
                                        XAttribute nilAttribute10 = issuerElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                        if (nilAttribute10 != null)
                                        {
                                            isNil9 = nilAttribute10.Value == "true";
                                        }
                                        if (isNil9 == false)
                                        {
                                            string issuerInstance = issuerElement.Value;
                                            certificateInstance.Issuer = issuerInstance;
                                        }
                                    }
                                    
                                    XElement passwordElement2 = sSLCertificatesElement.Element(XName.Get("Password", "http://schemas.microsoft.com/windowsazure"));
                                    if (passwordElement2 != null && passwordElement2.IsEmpty == false)
                                    {
                                        bool isNil10 = false;
                                        XAttribute nilAttribute11 = passwordElement2.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                        if (nilAttribute11 != null)
                                        {
                                            isNil10 = nilAttribute11.Value == "true";
                                        }
                                        if (isNil10 == false)
                                        {
                                            string passwordInstance = passwordElement2.Value;
                                            certificateInstance.Password = passwordInstance;
                                        }
                                    }
                                    
                                    XElement pfxBlobElement2 = sSLCertificatesElement.Element(XName.Get("PfxBlob", "http://schemas.microsoft.com/windowsazure"));
                                    if (pfxBlobElement2 != null && pfxBlobElement2.IsEmpty == false)
                                    {
                                        bool isNil11 = false;
                                        XAttribute nilAttribute12 = pfxBlobElement2.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                        if (nilAttribute12 != null)
                                        {
                                            isNil11 = nilAttribute12.Value == "true";
                                        }
                                        if (isNil11 == false)
                                        {
                                            byte[] pfxBlobInstance = Convert.FromBase64String(pfxBlobElement2.Value);
                                            certificateInstance.PfxBlob = pfxBlobInstance;
                                        }
                                    }
                                    
                                    XElement selfLinkElement = sSLCertificatesElement.Element(XName.Get("SelfLink", "http://schemas.microsoft.com/windowsazure"));
                                    if (selfLinkElement != null && selfLinkElement.IsEmpty == false)
                                    {
                                        bool isNil12 = false;
                                        XAttribute nilAttribute13 = selfLinkElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                        if (nilAttribute13 != null)
                                        {
                                            isNil12 = nilAttribute13.Value == "true";
                                        }
                                        if (isNil12 == false)
                                        {
                                            Uri selfLinkInstance = TypeConversion.TryParseUri(selfLinkElement.Value);
                                            certificateInstance.SelfLinkUri = selfLinkInstance;
                                        }
                                    }
                                    
                                    XElement siteNameElement = sSLCertificatesElement.Element(XName.Get("SiteName", "http://schemas.microsoft.com/windowsazure"));
                                    if (siteNameElement != null && siteNameElement.IsEmpty == false)
                                    {
                                        bool isNil13 = false;
                                        XAttribute nilAttribute14 = siteNameElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                        if (nilAttribute14 != null)
                                        {
                                            isNil13 = nilAttribute14.Value == "true";
                                        }
                                        if (isNil13 == false)
                                        {
                                            string siteNameInstance = siteNameElement.Value;
                                            certificateInstance.SiteName = siteNameInstance;
                                        }
                                    }
                                    
                                    XElement subjectNameElement = sSLCertificatesElement.Element(XName.Get("SubjectName", "http://schemas.microsoft.com/windowsazure"));
                                    if (subjectNameElement != null && subjectNameElement.IsEmpty == false)
                                    {
                                        bool isNil14 = false;
                                        XAttribute nilAttribute15 = subjectNameElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                        if (nilAttribute15 != null)
                                        {
                                            isNil14 = nilAttribute15.Value == "true";
                                        }
                                        if (isNil14 == false)
                                        {
                                            string subjectNameInstance = subjectNameElement.Value;
                                            certificateInstance.SubjectName = subjectNameInstance;
                                        }
                                    }
                                    
                                    XElement thumbprintElement4 = sSLCertificatesElement.Element(XName.Get("Thumbprint", "http://schemas.microsoft.com/windowsazure"));
                                    if (thumbprintElement4 != null && thumbprintElement4.IsEmpty == false)
                                    {
                                        bool isNil15 = false;
                                        XAttribute nilAttribute16 = thumbprintElement4.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                        if (nilAttribute16 != null)
                                        {
                                            isNil15 = nilAttribute16.Value == "true";
                                        }
                                        if (isNil15 == false)
                                        {
                                            string thumbprintInstance2 = thumbprintElement4.Value;
                                            certificateInstance.Thumbprint = thumbprintInstance2;
                                        }
                                    }
                                    
                                    XElement toDeleteElement2 = sSLCertificatesElement.Element(XName.Get("ToDelete", "http://schemas.microsoft.com/windowsazure"));
                                    if (toDeleteElement2 != null && toDeleteElement2.IsEmpty == false && string.IsNullOrEmpty(toDeleteElement2.Value) == false)
                                    {
                                        bool isNil16 = false;
                                        XAttribute nilAttribute17 = toDeleteElement2.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                        if (nilAttribute17 != null)
                                        {
                                            isNil16 = nilAttribute17.Value == "true";
                                        }
                                        if (isNil16 == false)
                                        {
                                            bool toDeleteInstance = bool.Parse(toDeleteElement2.Value);
                                            certificateInstance.IsToBeDeleted = toDeleteInstance;
                                        }
                                    }
                                    
                                    XElement validElement = sSLCertificatesElement.Element(XName.Get("Valid", "http://schemas.microsoft.com/windowsazure"));
                                    if (validElement != null && validElement.IsEmpty == false && string.IsNullOrEmpty(validElement.Value) == false)
                                    {
                                        bool isNil17 = false;
                                        XAttribute nilAttribute18 = validElement.Attribute(XName.Get("nil", "http://www.w3.org/2001/XMLSchema-instance"));
                                        if (nilAttribute18 != null)
                                        {
                                            isNil17 = nilAttribute18.Value == "true";
                                        }
                                        if (isNil17 == false)
                                        {
                                            bool validInstance = bool.Parse(validElement.Value);
                                            certificateInstance.IsValid = validInstance;
                                        }
                                    }
                                }
                            }
                        }
                        
                        XElement selfLinkElement2 = siteElement2.Element(XName.Get("SelfLink", "http://schemas.microsoft.com/windowsazure"));
                        if (selfLinkElement2 != null && selfLinkElement2.IsEmpty == false)
                        {
                            Uri selfLinkInstance2 = TypeConversion.TryParseUri(selfLinkElement2.Value);
                            webSiteInstance.Uri = selfLinkInstance2;
                        }
                        
                        XElement serverFarmElement2 = siteElement2.Element(XName.Get("ServerFarm", "http://schemas.microsoft.com/windowsazure"));
                        if (serverFarmElement2 != null && serverFarmElement2.IsEmpty == false)
                        {
                            string serverFarmInstance = serverFarmElement2.Value;
                            webSiteInstance.ServerFarm = serverFarmInstance;
                        }
                        
                        XElement siteModeElement2 = siteElement2.Element(XName.Get("SiteMode", "http://schemas.microsoft.com/windowsazure"));
                        if (siteModeElement2 != null && siteModeElement2.IsEmpty == false)
                        {
                            WebSiteMode siteModeInstance = ((WebSiteMode)Enum.Parse(typeof(WebSiteMode), siteModeElement2.Value, true));
                            webSiteInstance.SiteMode = siteModeInstance;
                        }
                        
                        XElement sitePropertiesElement = siteElement2.Element(XName.Get("SiteProperties", "http://schemas.microsoft.com/windowsazure"));
                        if (sitePropertiesElement != null && sitePropertiesElement.IsEmpty == false)
                        {
                            WebSite.WebSiteProperties sitePropertiesInstance = new WebSite.WebSiteProperties();
                            webSiteInstance.SiteProperties = sitePropertiesInstance;
                            
                            XElement appSettingsSequenceElement = sitePropertiesElement.Element(XName.Get("AppSettings", "http://schemas.microsoft.com/windowsazure"));
                            if (appSettingsSequenceElement != null && appSettingsSequenceElement.IsEmpty == false)
                            {
                                foreach (XElement appSettingsElement in appSettingsSequenceElement.Elements(XName.Get("NameValuePair", "http://schemas.microsoft.com/windowsazure")))
                                {
                                    string appSettingsKey = appSettingsElement.Element(XName.Get("Name", "http://schemas.microsoft.com/windowsazure")).Value;
                                    string appSettingsValue = appSettingsElement.Element(XName.Get("Value", "http://schemas.microsoft.com/windowsazure")).Value;
                                    sitePropertiesInstance.AppSettings.Add(appSettingsKey, appSettingsValue);
                                }
                            }
                            
                            XElement metadataSequenceElement = sitePropertiesElement.Element(XName.Get("Metadata", "http://schemas.microsoft.com/windowsazure"));
                            if (metadataSequenceElement != null && metadataSequenceElement.IsEmpty == false)
                            {
                                foreach (XElement metadataElement in metadataSequenceElement.Elements(XName.Get("NameValuePair", "http://schemas.microsoft.com/windowsazure")))
                                {
                                    string metadataKey = metadataElement.Element(XName.Get("Name", "http://schemas.microsoft.com/windowsazure")).Value;
                                    string metadataValue = metadataElement.Element(XName.Get("Value", "http://schemas.microsoft.com/windowsazure")).Value;
                                    sitePropertiesInstance.Metadata.Add(metadataKey, metadataValue);
                                }
                            }
                            
                            XElement propertiesSequenceElement = sitePropertiesElement.Element(XName.Get("Properties", "http://schemas.microsoft.com/windowsazure"));
                            if (propertiesSequenceElement != null && propertiesSequenceElement.IsEmpty == false)
                            {
                                foreach (XElement propertiesElement in propertiesSequenceElement.Elements(XName.Get("NameValuePair", "http://schemas.microsoft.com/windowsazure")))
                                {
                                    string propertiesKey = propertiesElement.Element(XName.Get("Name", "http://schemas.microsoft.com/windowsazure")).Value;
                                    string propertiesValue = propertiesElement.Element(XName.Get("Value", "http://schemas.microsoft.com/windowsazure")).Value;
                                    sitePropertiesInstance.Properties.Add(propertiesKey, propertiesValue);
                                }
                            }
                        }
                        
                        XElement stateElement2 = siteElement2.Element(XName.Get("State", "http://schemas.microsoft.com/windowsazure"));
                        if (stateElement2 != null && stateElement2.IsEmpty == false)
                        {
                            string stateInstance = stateElement2.Value;
                            webSiteInstance.State = stateInstance;
                        }
                        
                        XElement usageStateElement = siteElement2.Element(XName.Get("UsageState", "http://schemas.microsoft.com/windowsazure"));
                        if (usageStateElement != null && usageStateElement.IsEmpty == false)
                        {
                            WebSiteUsageState usageStateInstance = ((WebSiteUsageState)Enum.Parse(typeof(WebSiteUsageState), usageStateElement.Value, true));
                            webSiteInstance.UsageState = usageStateInstance;
                        }
                        
                        XElement webSpaceElement = siteElement2.Element(XName.Get("WebSpace", "http://schemas.microsoft.com/windowsazure"));
                        if (webSpaceElement != null && webSpaceElement.IsEmpty == false)
                        {
                            string webSpaceInstance = webSpaceElement.Value;
                            webSiteInstance.WebSpace = webSpaceInstance;
                        }
                    }
                    
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        Tracing.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// You can update the config settings for a web site by issuing an
        /// HTTP PUT with a request body containing the updated settings.
        /// (see http://msdn.microsoft.com/en-us/library/windowsazure/dn166985.aspx
        /// for more information)
        /// </summary>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Update Configuration Web Site
        /// operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public async System.Threading.Tasks.Task<OperationResponse> UpdateConfigurationAsync(string webSpaceName, string webSiteName, WebSiteUpdateConfigurationParameters parameters, CancellationToken cancellationToken)
        {
            // Validate
            if (webSpaceName == null)
            {
                throw new ArgumentNullException("webSpaceName");
            }
            if (webSiteName == null)
            {
                throw new ArgumentNullException("webSiteName");
            }
            if (parameters == null)
            {
                throw new ArgumentNullException("parameters");
            }
            
            // Tracing
            bool shouldTrace = CloudContext.Configuration.Tracing.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = Tracing.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("webSpaceName", webSpaceName);
                tracingParameters.Add("webSiteName", webSiteName);
                tracingParameters.Add("parameters", parameters);
                Tracing.Enter(invocationId, this, "UpdateConfigurationAsync", tracingParameters);
            }
            
            // Construct URL
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            string url = "/" + this.Client.Credentials.SubscriptionId.Trim() + "/services/WebSpaces/" + webSpaceName.Trim() + "/sites/" + webSiteName.Trim() + "/config";
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Put;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-version", "2013-08-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Serialize Request
                string requestContent = null;
                XDocument requestDoc = new XDocument();
                
                XElement siteConfigElement = new XElement(XName.Get("SiteConfig", "http://schemas.microsoft.com/windowsazure"));
                requestDoc.Add(siteConfigElement);
                
                if (parameters.AppSettings != null)
                {
                    XElement appSettingsDictionaryElement = new XElement(XName.Get("AppSettings", "http://schemas.microsoft.com/windowsazure"));
                    foreach (KeyValuePair<string, string> pair in parameters.AppSettings)
                    {
                        string appSettingsKey = pair.Key;
                        string appSettingsValue = pair.Value;
                        XElement appSettingsElement = new XElement(XName.Get("NameValuePair", "http://schemas.microsoft.com/windowsazure"));
                        appSettingsDictionaryElement.Add(appSettingsElement);
                        
                        XElement appSettingsKeyElement = new XElement(XName.Get("Name", "http://schemas.microsoft.com/windowsazure"));
                        appSettingsKeyElement.Value = appSettingsKey;
                        appSettingsElement.Add(appSettingsKeyElement);
                        
                        XElement appSettingsValueElement = new XElement(XName.Get("Value", "http://schemas.microsoft.com/windowsazure"));
                        appSettingsValueElement.Value = appSettingsValue;
                        appSettingsElement.Add(appSettingsValueElement);
                    }
                    siteConfigElement.Add(appSettingsDictionaryElement);
                }
                
                if (parameters.ConnectionStrings != null)
                {
                    XElement connectionStringsSequenceElement = new XElement(XName.Get("ConnectionStrings", "http://schemas.microsoft.com/windowsazure"));
                    foreach (WebSiteUpdateConfigurationParameters.ConnectionStringInfo connectionStringsItem in parameters.ConnectionStrings)
                    {
                        XElement connStringInfoElement = new XElement(XName.Get("ConnStringInfo", "http://schemas.microsoft.com/windowsazure"));
                        connectionStringsSequenceElement.Add(connStringInfoElement);
                        
                        if (connectionStringsItem.ConnectionString != null)
                        {
                            XElement connectionStringElement = new XElement(XName.Get("ConnectionString", "http://schemas.microsoft.com/windowsazure"));
                            connectionStringElement.Value = connectionStringsItem.ConnectionString;
                            connStringInfoElement.Add(connectionStringElement);
                        }
                        
                        if (connectionStringsItem.Name != null)
                        {
                            XElement nameElement = new XElement(XName.Get("Name", "http://schemas.microsoft.com/windowsazure"));
                            nameElement.Value = connectionStringsItem.Name;
                            connStringInfoElement.Add(nameElement);
                        }
                        
                        if (connectionStringsItem.Type != null)
                        {
                            XElement typeElement = new XElement(XName.Get("Type", "http://schemas.microsoft.com/windowsazure"));
                            typeElement.Value = connectionStringsItem.Type;
                            connStringInfoElement.Add(typeElement);
                        }
                    }
                    siteConfigElement.Add(connectionStringsSequenceElement);
                }
                
                if (parameters.DefaultDocuments != null)
                {
                    XElement defaultDocumentsSequenceElement = new XElement(XName.Get("DefaultDocuments", "http://schemas.microsoft.com/windowsazure"));
                    foreach (string defaultDocumentsItem in parameters.DefaultDocuments)
                    {
                        XElement defaultDocumentsItemElement = new XElement(XName.Get("string", "http://schemas.microsoft.com/2003/10/Serialization/Arrays"));
                        defaultDocumentsItemElement.Value = defaultDocumentsItem;
                        defaultDocumentsSequenceElement.Add(defaultDocumentsItemElement);
                    }
                    siteConfigElement.Add(defaultDocumentsSequenceElement);
                }
                
                if (parameters.DetailedErrorLoggingEnabled != null)
                {
                    XElement detailedErrorLoggingEnabledElement = new XElement(XName.Get("DetailedErrorLoggingEnabled", "http://schemas.microsoft.com/windowsazure"));
                    detailedErrorLoggingEnabledElement.Value = parameters.DetailedErrorLoggingEnabled.ToString().ToLower();
                    siteConfigElement.Add(detailedErrorLoggingEnabledElement);
                }
                
                if (parameters.DocumentRoot != null)
                {
                    XElement documentRootElement = new XElement(XName.Get("DocumentRoot", "http://schemas.microsoft.com/windowsazure"));
                    documentRootElement.Value = parameters.DocumentRoot;
                    siteConfigElement.Add(documentRootElement);
                }
                
                if (parameters.HandlerMappings != null)
                {
                    XElement handlerMappingsSequenceElement = new XElement(XName.Get("HandlerMappings", "http://schemas.microsoft.com/windowsazure"));
                    foreach (WebSiteUpdateConfigurationParameters.HandlerMapping handlerMappingsItem in parameters.HandlerMappings)
                    {
                        XElement handlerMappingElement = new XElement(XName.Get("HandlerMapping", "http://schemas.microsoft.com/windowsazure"));
                        handlerMappingsSequenceElement.Add(handlerMappingElement);
                        
                        if (handlerMappingsItem.Arguments != null)
                        {
                            XElement argumentsElement = new XElement(XName.Get("Arguments", "http://schemas.microsoft.com/windowsazure"));
                            argumentsElement.Value = handlerMappingsItem.Arguments;
                            handlerMappingElement.Add(argumentsElement);
                        }
                        
                        if (handlerMappingsItem.Extension != null)
                        {
                            XElement extensionElement = new XElement(XName.Get("Extension", "http://schemas.microsoft.com/windowsazure"));
                            extensionElement.Value = handlerMappingsItem.Extension;
                            handlerMappingElement.Add(extensionElement);
                        }
                        
                        if (handlerMappingsItem.ScriptProcessor != null)
                        {
                            XElement scriptProcessorElement = new XElement(XName.Get("ScriptProcessor", "http://schemas.microsoft.com/windowsazure"));
                            scriptProcessorElement.Value = handlerMappingsItem.ScriptProcessor;
                            handlerMappingElement.Add(scriptProcessorElement);
                        }
                    }
                    siteConfigElement.Add(handlerMappingsSequenceElement);
                }
                
                if (parameters.HttpLoggingEnabled != null)
                {
                    XElement httpLoggingEnabledElement = new XElement(XName.Get("HttpLoggingEnabled", "http://schemas.microsoft.com/windowsazure"));
                    httpLoggingEnabledElement.Value = parameters.HttpLoggingEnabled.ToString().ToLower();
                    siteConfigElement.Add(httpLoggingEnabledElement);
                }
                
                if (parameters.LogsDirectorySizeLimit != null)
                {
                    XElement logsDirectorySizeLimitElement = new XElement(XName.Get("LogsDirectorySizeLimit", "http://schemas.microsoft.com/windowsazure"));
                    logsDirectorySizeLimitElement.Value = parameters.LogsDirectorySizeLimit.ToString();
                    siteConfigElement.Add(logsDirectorySizeLimitElement);
                }
                
                if (parameters.ManagedPipelineMode != null)
                {
                    XElement managedPipelineModeElement = new XElement(XName.Get("ManagedPipelineMode", "http://schemas.microsoft.com/windowsazure"));
                    managedPipelineModeElement.Value = parameters.ManagedPipelineMode.ToString();
                    siteConfigElement.Add(managedPipelineModeElement);
                }
                
                if (parameters.Metadata != null)
                {
                    XElement metadataDictionaryElement = new XElement(XName.Get("Metadata", "http://schemas.microsoft.com/windowsazure"));
                    foreach (KeyValuePair<string, string> pair2 in parameters.Metadata)
                    {
                        string metadataKey = pair2.Key;
                        string metadataValue = pair2.Value;
                        XElement metadataElement = new XElement(XName.Get("NameValuePair", "http://schemas.microsoft.com/windowsazure"));
                        metadataDictionaryElement.Add(metadataElement);
                        
                        XElement metadataKeyElement = new XElement(XName.Get("Name", "http://schemas.microsoft.com/windowsazure"));
                        metadataKeyElement.Value = metadataKey;
                        metadataElement.Add(metadataKeyElement);
                        
                        XElement metadataValueElement = new XElement(XName.Get("Value", "http://schemas.microsoft.com/windowsazure"));
                        metadataValueElement.Value = metadataValue;
                        metadataElement.Add(metadataValueElement);
                    }
                    siteConfigElement.Add(metadataDictionaryElement);
                }
                
                if (parameters.NetFrameworkVersion != null)
                {
                    XElement netFrameworkVersionElement = new XElement(XName.Get("NetFrameworkVersion", "http://schemas.microsoft.com/windowsazure"));
                    netFrameworkVersionElement.Value = parameters.NetFrameworkVersion;
                    siteConfigElement.Add(netFrameworkVersionElement);
                }
                
                if (parameters.NumberOfWorkers != null)
                {
                    XElement numberOfWorkersElement = new XElement(XName.Get("NumberOfWorkers", "http://schemas.microsoft.com/windowsazure"));
                    numberOfWorkersElement.Value = parameters.NumberOfWorkers.ToString();
                    siteConfigElement.Add(numberOfWorkersElement);
                }
                
                if (parameters.PhpVersion != null)
                {
                    XElement phpVersionElement = new XElement(XName.Get("PhpVersion", "http://schemas.microsoft.com/windowsazure"));
                    phpVersionElement.Value = parameters.PhpVersion;
                    siteConfigElement.Add(phpVersionElement);
                }
                
                if (parameters.PublishingPassword != null)
                {
                    XElement publishingPasswordElement = new XElement(XName.Get("PublishingPassword", "http://schemas.microsoft.com/windowsazure"));
                    publishingPasswordElement.Value = parameters.PublishingPassword;
                    siteConfigElement.Add(publishingPasswordElement);
                }
                
                if (parameters.PublishingUserName != null)
                {
                    XElement publishingUsernameElement = new XElement(XName.Get("PublishingUsername", "http://schemas.microsoft.com/windowsazure"));
                    publishingUsernameElement.Value = parameters.PublishingUserName;
                    siteConfigElement.Add(publishingUsernameElement);
                }
                
                if (parameters.RemoteDebuggingEnabled != null)
                {
                    XElement remoteDebuggingEnabledElement = new XElement(XName.Get("RemoteDebuggingEnabled", "http://schemas.microsoft.com/windowsazure"));
                    remoteDebuggingEnabledElement.Value = parameters.RemoteDebuggingEnabled.ToString().ToLower();
                    siteConfigElement.Add(remoteDebuggingEnabledElement);
                }
                
                XElement remoteDebuggingVersionElement = new XElement(XName.Get("RemoteDebuggingVersion", "http://schemas.microsoft.com/windowsazure"));
                remoteDebuggingVersionElement.Value = parameters.RemoteDebuggingVersion.ToString();
                siteConfigElement.Add(remoteDebuggingVersionElement);
                
                if (parameters.RequestTracingEnabled != null)
                {
                    XElement requestTracingEnabledElement = new XElement(XName.Get("RequestTracingEnabled", "http://schemas.microsoft.com/windowsazure"));
                    requestTracingEnabledElement.Value = parameters.RequestTracingEnabled.ToString().ToLower();
                    siteConfigElement.Add(requestTracingEnabledElement);
                }
                
                if (parameters.RequestTracingExpirationTime != null)
                {
                    XElement requestTracingExpirationTimeElement = new XElement(XName.Get("RequestTracingExpirationTime", "http://schemas.microsoft.com/windowsazure"));
                    requestTracingExpirationTimeElement.Value = parameters.RequestTracingExpirationTime.ToString();
                    siteConfigElement.Add(requestTracingExpirationTimeElement);
                }
                
                if (parameters.ScmType != null)
                {
                    XElement scmTypeElement = new XElement(XName.Get("ScmType", "http://schemas.microsoft.com/windowsazure"));
                    scmTypeElement.Value = parameters.ScmType;
                    siteConfigElement.Add(scmTypeElement);
                }
                
                if (parameters.Use32BitWorkerProcess != null)
                {
                    XElement use32BitWorkerProcessElement = new XElement(XName.Get("Use32BitWorkerProcess", "http://schemas.microsoft.com/windowsazure"));
                    use32BitWorkerProcessElement.Value = parameters.Use32BitWorkerProcess.ToString().ToLower();
                    siteConfigElement.Add(use32BitWorkerProcessElement);
                }
                
                if (parameters.WebSocketsEnabled != null)
                {
                    XElement webSocketsEnabledElement = new XElement(XName.Get("WebSocketsEnabled", "http://schemas.microsoft.com/windowsazure"));
                    webSocketsEnabledElement.Value = parameters.WebSocketsEnabled.ToString().ToLower();
                    siteConfigElement.Add(webSocketsEnabledElement);
                }
                
                requestContent = requestDoc.ToString();
                httpRequest.Content = new StringContent(requestContent, Encoding.UTF8);
                httpRequest.Content.Headers.ContentType = new MediaTypeHeaderValue("application/xml");
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        Tracing.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        Tracing.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, requestContent, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false), CloudExceptionType.Xml);
                        if (shouldTrace)
                        {
                            Tracing.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    OperationResponse result = null;
                    result = new OperationResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        Tracing.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
    }
}
